// BVQXPluginInterface.h : general header file for BrainVoyagerQX plugin support
//

#ifndef BVQXPLUGININTERFACE_EXTERNWRAPPER_H
#define BVQXPLUGININTERFACE_EXTERNWRAPPER_H

#include "Plugin_FMR_Header.h"
#include "Plugin_DMR_Header.h"
#include "Plugin_VMR_Header.h"
#include "Plugin_VTC_Header.h"
#include "Plugin_NR_VMP_Header.h"
#include "Plugin_AR_VMP_Header.h"
#include "Plugin_PRT_Header.h"
#include "Plugin_VOI_Header.h"
#include "Plugin_GLM_Header.h"
#include "Plugin_SRF_Header.h"
#include "Plugin_MTC_Header.h"
#include "Plugin_SMP_Header.h"
#include "Plugin_SSM_Header.h"
#include "Plugin_POI_Header.h"

#include "global.h"

// make sure that "MAKE_PLUGIN" is defined here or as a preprocessor definition (this is only important on Windows)
//
#define MAKE_PLUGIN

#if defined OS_WIN32
#ifdef MAKE_PLUGIN
#define PLUGIN_ACCESS __declspec(dllexport)
#else
#define PLUGIN_ACCESS __declspec(dllimport)
#endif
#else
// for all other platforms PLUGIN_ACCESS is empty (not needed)
#define PLUGIN_ACCESS
#endif


class PLUGIN_ACCESS BVQXPluginInterface
{
public:	 
	BVQXPluginInterface() {};
	virtual ~BVQXPluginInterface() {};

	// use this method to execute the plugin (your essential code)
	//
	virtual bool execute() = 0;

	// this method has been introduced with BVQX 2.1 for GUI plugin support: this function will be called in case of GUI plugins once, the execute() method may be called repeatedly from GUI script
	//
	virtual bool initPlugin() = 0;
};


//
// These are typedef's for dynamic access of QX functions
//

// general functions
extern void qxGetVersionOfBrainVoyagerQX(int *, int *, int *);

extern void qxGetBrainVoyagerQXPath(char *);

extern void qxGetPluginsPath(char *);



// input / output
extern void qxShowLogPane(); // v2.0;

extern void qxShowLogTab();

extern void qxLogText(const char *);

extern void qxMessageBox(const char *);

extern int qxYesNoMessageBox(const char *);

extern int qxGetIntegerInput(const char *, int, int, int);

extern float qxGetFloatInput(const char *, float, float, float);

extern int qxGetTextInput(const char *, const char *, char *);

extern int qxGetOpenFileName(const char *, const char *, const char *, char *);

extern int qxGetSaveFileName(const char *, const char *, const char *, char *);


// GUI
extern void qxUpdateActiveWindow();

extern void qxShowBusyCursor();

extern void qxStopBusyCursor();


// Docs
extern int qxGetNumberOfDocsInWorkspace();

extern int qxGetDocumentType(int);

extern char * qxGetDocumentTypeName(int);

extern int qxGetIDOfCurrentDocument();

extern void qxSetAsCurrentDocument(int);

extern int qxCalculateIAMatrix(struct FMR_ScannerPosition_Header *, struct VMR_ScannerPosition_Header *, struct VMR_PastTransformations_Header *, float *, float *, float *, float *, float *, float *, float *, float *);

extern int qxGetTRFParametersFromVolumeToolsDlg(float *, float *, float *, float *, float *, float *, float *, float *, float *);


// FMR-STC
extern int qxGetHeaderOfCurrentFMR(struct FMR_Header *);

extern int qxSetHeaderOfCurrentFMR(struct FMR_Header *);

extern float ** qxGetSTCFloatDataOfCurrentFMR();  // NEW in BVQX v2.0 - use to get float ptr to whole STC data

extern short ** qxGetSTCIntDataOfCurrentFMR();    // NEW in BVQX v2.0 - use to get short int ptr to whole STC data

extern short ** qxGetSTCDataOfCurrentFMR();       // deprecated in BVQX 2.0 -> use new fns for int or float

extern int qxSaveFMR(const char *);

extern int qxSaveFMRAndSTC(const char *, const char *);

extern unsigned int qxGetScannerPositionHeaderOfCurrentFMR(struct FMR_ScannerPosition_Header *);

extern unsigned int qxSetScannerPositionHeaderOfCurrentFMR(struct FMR_ScannerPosition_Header *);


// DMR-DWI
extern int qxGetHeaderOfCurrentDMR(struct DMR_Header *);

extern int qxSetHeaderOfCurrentDMR(struct DMR_Header *);

extern float ** qxGetDWIFloatDataOfCurrentDMR();     // NEW in BVQX v2.0 - use to get float ptr to whole STC data

extern short ** qxGetDWIIntDataOfCurrentDMR();       // NEW in BVQX v2.0 - use to get short int ptr to whole STC data

extern short ** qxGetDWIDataOfCurrentDMR();          // deprecated in BVQX 2.0 -> use new fns for int or float

extern int qxConvertDWIFormatOfCurrentDMR(int, int); // works for both int and float data type

extern int qxSaveDMR(const char *);

extern int qxSaveDMRAndDWI(const char *, const char *);


// VMR
extern unsigned char * qxGetCurrentVMR(int *, int *, int *);

extern int qxGetMainHeaderOfCurrentVMR(struct VMR_Header *);

extern int qxSetMainHeaderOfCurrentVMR(struct VMR_Header *);

extern int qxGetScannerPositionHeaderOfCurrentVMR(struct VMR_ScannerPosition_Header *);

extern int qxSetScannerPositionHeaderOfCurrentVMR(struct VMR_ScannerPosition_Header *);

extern int qxGetPastTransformationsHeaderOfCurrentVMR(struct VMR_PastTransformations_Header *);

extern int qxSetPastTransformationsHeaderOfCurrentVMR(struct VMR_PastTransformations_Header *);

extern int qxReframeCurrentVMR(int, int, int);

extern int qxGetCrossPositionOfCurrentVMR(int *, int *, int *);


// VMP - native resolution
extern float ** qxGetNRVMPsOfCurrentVMR(struct NR_VMPs_Header *);

extern float * qxGetNRVMPOfCurrentVMR(int, struct NR_VMP_Header *);

extern int qxSetNRVMPParametersOfCurrentVMR(int, struct NR_VMP_Header *);

extern int qxLoadNRVMPsToCurrentVMR(const char *);

extern void qxSaveNRVMPsOfCurrentVMR(const char *);

extern int qxCreateNRVMPsForCurrentVMR(int, int, int, struct NR_VMPs_Header *);

extern void qxDeleteNRVMPsOfCurrentVMR();

extern int qxSetNRVMPsParamsDisplayRange(struct NR_VMPs_Header *vmps_header);


// VMP - anatomical resolution
extern float ** qxGetARVMPsOfCurrentVMR(struct AR_VMPs_Header *);

extern float * qxGetARVMPOfCurrentVMR(int, struct AR_VMP_Header *);

extern int qxSetARVMPParametersOfCurrentVMR(int, struct AR_VMP_Header *);

extern int qxLoadARVMPsToCurrentVMR(const char *);

extern void qxSaveARVMPsOfCurrentVMR(const char *);

extern int qxCreateARVMPsForCurrentVMR(int , struct AR_VMPs_Header *);

extern void qxDeleteARVMPsOfCurrentVMR();


// VTC
extern int qxLinkVTCToCurrentVMR(const char *, int);

extern int qxCreateVTCForCurrentVMR(int, struct VTC_Header *);    // if NULL ptr to VTC_Header, float data created as default - fill header (data type) for more control

extern int qxGetHeaderOfVTCOfCurrentVMR(struct VTC_Header *);     // NEW in BVQX v2.0

extern int qxSetHeaderOfVTCOfCurrentVMR(struct VTC_Header *);

extern int qxSetVTCParametersOfCurrentVMR(struct VTC_Header *);   // deprecated -> use fn above

extern float * qxGetVTCFloatDataOfCurrentVMR();                   // NEW in BVQX v2.0 - use to get float ptr to whole VTC data

extern short * qxGetVTCIntDataOfCurrentVMR();                     // NEW in BVQX v2.0 - use to get short int ptr to whole VTC data

extern short * qxGetVTCOfCurrentVMR(struct VTC_Header *);         // deprecated in BVQX 2.0 -> use fns above

extern void qxSaveVTCOfCurrentVMR(const char *);


// Multi-VTC
extern int qxClearMultiVTCAnalysis();

extern int qxAddVTCRefForMultiVTCAnalysis(const char *);

extern int qxGetVoxelTimeCourseOfVTC(int , int , float *);                 // CHANGED in BVQX v2.0 - provide float ptr, float time series always returned, even if VTC uses short int data type

extern int qxGetVoxelTimeCourseOfVTCs(int , float *);                      // CHANGED in BVQX 2.0 - see above

extern int qxGetVoxelCoordsTimeCourseOfVTC(int , int , int, int, float *); // CHANGED in BVQX 2.0 - see above

extern int qxGetVoxelCoordsTimeCourseOfVTCs(int , int, int, float *);      // CHANGED in BVQX 2.0 - see above


// GLM
extern float * qxGetGLMOfCurrentVMR(struct GLM_Header *);


// MSK
extern int qxLoadMaskToCurrentVMR(const char *);

extern unsigned char * qxGetMaskOfCurrentVMR();


// SRF
extern int qxGetScene(struct SCENE_Header *);

extern int qxGetCurrentSRF(struct SRF_Header *);

extern void qxReleaseSRFAccess(struct SRF_Header *);

extern void qxSaveCurrentSRF(const char *);

extern int qxSetCurrentSRF(int);


// SMP
extern int qxGetSMPsOfCurrentSRF(struct SMPs_Header *);

extern float * qxGetSMPOfCurrentSRF(int, struct SMP_Header *);

extern int qxSetSMPParametersOfCurrentSRF(int, struct SMP_Header *);

extern int qxLoadSMPsToCurrentSRF(const char *);

extern void qxSaveSMPsOfCurrentSRF(const char *);

extern int qxCreateSMPForCurrentSRF();

extern void qxDeleteSMPsOfCurrentSRF();

extern void qxOverlaySMPsOnSRF();


// MTC
extern int qxLinkMTCToCurrentSRF(const char *, int);

extern int qxCreateMTCForCurrentSRF(int);

extern float ** qxGetMTCOfCurrentSRF(struct MTC_Header *);

extern int qxSetMTCParametersOfCurrentSRF(struct MTC_Header *);

extern void qxSaveMTCOfCurrentSRF(const char *);

// Multi-MTC (file-based read access)
extern int qxClearMultiMTCRefsOfCurrentSRF();

extern int qxAddMultiMTCRefToCurrentSRF(const char *);

extern int qxGetVertexTimeCourseOfMTC(int , int , float *);

extern int qxGetVertexTimeCourseOfAllMTCs(int , float *);


// SSM
extern int *qxGetSSMOfCurrentSRF(struct SSM_Header *);

extern int qxLoadSSMToCurrentSRF(const char *);

extern void qxSaveSSMOfCurrentSRF(const char *);

extern int qxCreateSSMForCurrentSRF();

extern void qxDeleteSSMOfCurrentSRF();


// PRT
extern void qxClearProtocol();

extern int qxLoadProtocol(const char *);

extern int qxSaveProtocol(const char *);

extern int qxGetProtocol(struct Protocol_Header *);

extern int qxSetProtocol(struct Protocol_Header *);

extern int qxGetProtocolCondition(int, struct ProtocolCondition_Header *);

extern int qxAddProtocolCondition(struct ProtocolCondition_Header *);

extern int qxAddProtocolConditionInterval(int, int, int);


// PRED, DM, HRF
extern int qxClearDesignMatrix();

extern int qxAddPredictor(const char *);

extern int qxSetPredictorValuesFromCondition(const char *, int, float);

extern int qxSetPredictorValuesFromConditionName(const char *, const char *, float);

extern int qxApplyTwoGammaHRFToPredictor(int);

extern int qxApplyTwoGammaHRFToPredictorWithName(const char *);

extern int qxApplyBoyntonHRFToPredictor(int);

extern int qxApplyBoyntonHRFToPredictorWithName(const char *);

extern float * qxGetPredictor(int);

extern float * qxGetPredictorByName(const char *);

extern int qxLoadSingleStudyGLMDesignMatrix(const char *);

extern int qxSaveSingleStudyGLMDesignMatrix(const char *);


// VOI
extern int qxLoadVOIs(const char *);

extern int qxSaveVOIs(const char *);

extern int qxGetVOIs(struct VOIs_Header *);

extern int * qxGetVOI(int, struct VOI_Header *);


// POI
extern int qxLoadPOIs(const char *);

extern int qxSavePOIs(const char *);

extern int qxGetPOIs(struct POIs_Header *);

extern int * qxGetPOI(int, struct POI_Header *);


// Access to (get, set) parameters for exchange with GUI scripts
extern int qxGetStringParameter(char *, char *);

extern int qxGetIntParameter(char *);

extern float qxGetFloatParameter(char *);

extern void qxSetStringParameter(char *, char *);

extern void qxSetIntParameter(char *, int);

extern void qxSetFloatParameter(char *, float);


// BVQX dialog access (used only internally, for GUI plugins, use new functions above) - deprecated, will be removed in future plugin version
extern int qxGetDlgStringParameter(int, const char *);

extern int qxGetDlgIntegerParameter(int);

extern float qxGetDlgFloatParameter(int);

extern void qxSetDlgStringOutputParameter(int, const char *);

extern void qxSetDlgIntegerOutputParameter(int, int);

extern void qxSetDlgFloatOutputParameter(int, float);

extern void qxLaunchICADialog();



// Locally defined functions
extern  bool InitQXAccess();
extern  bool checkVersionGreaterEqualVersion(int MajorA, int MinorA, int SubMinorA, int MajorB, int MinorB, int SubMinorB);


#endif // BVQXPLUGININTERFACE_H
