#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string>

#include "BVQXPluginInterfaceExtern.h"
#include "bvqx_io.h"
#include "SRF_Header.h"
#include "MTC_Header.h"
#include "SMP_Header.h"
#include "POI_Header.h"
#include "convertbvqxstructs.h"
#include "constants.h"

float** create2dArray(int rows, int cols) {

	float** array = new float*[rows];

	for (int row=0; row<rows; row++) {

		array[row] = new float[cols];

	}

	return array;

}

void delete2dArray(float **ar, int rows, int cols) {

	for (int row=0; row<rows; row++) {

		delete [] ar[row];

	}

	delete [] ar;

}

int** create2dIntArray(int rows, int cols) {

	int** array = new int*[rows];

	for (int row=0; row<rows; row++) {

		array[row] = new int[cols];

	}

	return array;
}

void delete2dIntArray(int **ar, int rows, int cols) {

	for (int row=0; row<rows; row++) {

		delete [] ar[row];

	}

	delete [] ar;
}


char *removeExtension(char *fileName){

	int ii; int strLength; int lout;
	char *outName ;
	if( fileName == NULL || (strLength=strlen(fileName)) == 0){
		outName = strdup("''") ; return outName;
	}
	int endIndex = 0;
	lout = 4 ;
	for( ii=0 ; ii < strLength ; ii++ ){
		switch( fileName[ii] ){
		case '.': endIndex = ii;
		default: lout++ ; break ;
		}
	}
	outName = (char *)calloc(1,lout) ;
	for( ii=0; ii < endIndex; ii++){
		outName[ii] = fileName[ii];
	}
	outName[ii]   = '\0' ;
	return outName;

}

char *exchangeExtension(char *currentFileName, char *newExtension) {

	int length;
	char *result;
	char *fileNameWithoutExt = removeExtension(currentFileName);
	length = strlen(fileNameWithoutExt);
	int extLength = strlen(newExtension);
	result = new char[length+extLength];
	strcpy(result, fileNameWithoutExt);
	strcat(result, newExtension);
	return result;

}

char *getStatMapStr(int statMapType) {

	switch(statMapType) {
		case BVQX_MAPTYPE_T_STAT:			return "T-test values";	break; // of curvature
		case BVQX_MAPTYPE_CORRELATION:		return "Correlation values"; break;
		case BVQX_MAPTYPE_CROSSCORRELATION:	return "Cross-correlation values"; break;
		case BVQX_MAPTYPE_F_STAT:			return "F-test values";	break;
		case BVQX_MAPTYPE_PSC:				return"Percent signal change values"; break;
		default:							return "Unknown map type";
			// toevoegen ICA, prob. maps, etc.
	}
}

srfStruct *getsrf(char *filename, bool exportfromdisk) {

	bool success = true;
	srfStruct *srf = new srfStruct;
	if (exportfromdisk) {
		success = readsrf(filename, srf);
	} else {
		SRF_Header *bvqxsrfhdr = new SRF_Header;
		success = (bool)qxGetCurrentSRF(bvqxsrfhdr);
		// now convert BV structure to superstructure
		if (success){
			srf = convertSRF(bvqxsrfhdr);
			srf->fileVersion = 3.5f;
		}
	}
	if (!success){
		srf->fileVersion = -1.0f;
		qxLogText("Reading of surface file did not succeed.");
	}
	return srf;
}


bool readsrf(char *srfFileName, srfStruct *srf) {

	bool readData = true;
	FILE *srfFile;
	int nrOfVertices;

	try {

		srfFile = fopen(srfFileName, "rb");
		int readSize;
		readSize = fread(&srf->fileVersion, 1, sizeof(float), srfFile);

		//Check for supported versions before continuing:
		if ((srf->fileVersion < 3)||(srf->fileVersion > 4))
		{
			qxMessageBox("SRF file version is not supported");
			return false;
		}

		readSize += fread(&srf->reserved, 1, sizeof(int), srfFile);
		readSize += fread(&srf->NrOfVertices, 1, sizeof(int), srfFile);
		nrOfVertices = srf->NrOfVertices;
		readSize += fread(&srf->NrOfTriangles, 1, sizeof(int), srfFile);
		readSize += fread(&srf->MeshCenterX, 1, sizeof(float), srfFile);
		readSize += fread(&srf->MeshCenterY, 1, sizeof(float), srfFile);
		readSize += fread(&srf->MeshCenterZ, 1, sizeof(float), srfFile);
		int totVertices = srf->NrOfVertices * sizeof(float);

		if (readData) {

			float *vertexX = new float[nrOfVertices];
			float *vertexY = new float[nrOfVertices];
			float *vertexZ = new float[nrOfVertices];
			float *normalX = new float[nrOfVertices];
			float *normalY = new float[nrOfVertices];
			float *normalZ = new float[nrOfVertices];

			readSize += fread(vertexX, 1, totVertices, srfFile);
			readSize += fread(vertexY, 1, totVertices, srfFile);
			readSize += fread(vertexZ, 1, totVertices, srfFile);
			readSize += fread(normalX, 1, totVertices, srfFile);
			readSize += fread(normalY, 1, totVertices, srfFile);
			readSize += fread(normalZ, 1, totVertices, srfFile);

			srf->VertexX = vertexX;
			srf->VertexY = vertexY;
			srf->VertexZ = vertexZ;
			srf->NormalX = normalX;
			srf->NormalY = normalY;
			srf->NormalZ = normalZ;

		} else {
			fseek(srfFile, (totVertices * 6), SEEK_CUR);
		}

		readSize += fread(&srf->ConvCurvRed, 1, sizeof(float), srfFile);
		readSize += fread(&srf->ConvCurvGreen, 1, sizeof(float), srfFile);
		readSize += fread(&srf->ConvCurvBlue, 1, sizeof(float), srfFile);
		readSize += fread(&srf->ConvCurvAlpha, 1, sizeof(float), srfFile);
		readSize += fread(&srf->ConcCurvRed, 1, sizeof(float), srfFile);
		readSize += fread(&srf->ConcCurvGreen, 1, sizeof(float), srfFile);
		readSize += fread(&srf->ConcCurvBlue, 1, sizeof(float), srfFile);
		readSize += fread(&srf->ConcCurvAlpha, 1, sizeof(float), srfFile);

		int *meshColors = new int[nrOfVertices];
		if (readData) {
			readSize += fread(meshColors, 1, (nrOfVertices * sizeof(int)), srfFile);
			srf->MeshColor = meshColors;
		} else {
			fseek(srfFile, (nrOfVertices * sizeof(int)), SEEK_CUR);
		}

		int vertexCounter;
		int nrOfNeighbours;
		neighbours *neighboursPtrContainer = new neighbours[nrOfVertices];
		int elemcounter = 0;
		for (vertexCounter = 0; vertexCounter < nrOfVertices; vertexCounter++) {

			readSize += fread(&nrOfNeighbours, 1, sizeof(int), srfFile);
			neighboursPtrContainer[vertexCounter].nrOfNeighbours = nrOfNeighbours;
			elemcounter++;
			int neighbSize = nrOfNeighbours * sizeof(int);
			elemcounter += nrOfNeighbours;
			if (readData){
				int *neighb = new int[nrOfNeighbours];
				readSize += fread(neighb, 1, neighbSize, srfFile);
				neighboursPtrContainer[vertexCounter].neighbourLst = neighb;
			} else {
				fseek(srfFile, neighbSize, SEEK_CUR);
			}
		}
		srf->totalneighbourelems = elemcounter;
		srf->neighb = neighboursPtrContainer;

		int totTri = srf->NrOfTriangles * 3;
		if (readData) {
			int *triangles = new int[totTri];
			readSize += fread(triangles, 1, (totTri  * sizeof(int)), srfFile);
			srf->triangleSequence = triangles;
		} else {
			fseek(srfFile, totTri, SEEK_CUR);
		}

		readSize += fread(&srf->nrOfTriangleStripElements, 1, sizeof(int), srfFile);
		if (readData) {
			int *trianglestrs = new int[srf->nrOfTriangleStripElements];
			readSize += fread(trianglestrs, 1, (srf->nrOfTriangleStripElements * sizeof(int)), srfFile);
		}

		fclose(srfFile);
		srf->origName = srfFileName;

	} catch (...) {
		char *txt = new char[512];
		sprintf(txt, "Error while reading surface file %s.", srfFileName);
		qxLogText(txt);
	}
	return srf;
}

mtcStruct *getmtc(char *filename, bool exportfromdisk) {

	bool success = true;
	mtcStruct *mtc = new mtcStruct;
	if (exportfromdisk) {
		success = readmtc(filename, mtc);
	} else {
		MTC_Header *bvqxmtchdr = new MTC_Header;
		float **data = qxGetMTCOfCurrentSRF(bvqxmtchdr); // requires that MTC is loaded in memory
		// now convert BV structure to superstructure
		if (data != NULL){
			mtc = convertMTC(bvqxmtchdr, data);
			mtc->fileVersion = 1.0f;
		} else {
			qxLogText("Could not get current mesh time course (MTC).\nPlease select the option 'Load in memory' when linking the MTC.");
			success = false;
		}
	}
	if (!success){
		mtc->fileVersion = -1.0f;
		qxLogText("Reading of mesh time course (MTC) file did not succeed.");
	}
	return mtc;
}

bool readmtc(char *mtcFileName, mtcStruct *mtc) {

	FILE *mtcFile;
	int readSize;
	bool readData = true;
	char *txt = new char[1024];
	mtc->origName = mtcFileName;

	try {

		mtcFile = fopen(mtcFileName,"rb");
		if (mtcFile == NULL) {
			qxMessageBox("File could not be accessed.\nPlease check if it is opened in other programs.");
		}

		readSize = fread(&mtc->fileVersion, 1, sizeof(int), mtcFile);

		//Check for supported versions before continuing:
		if (mtc->fileVersion > 1)
		{
			qxMessageBox("SRF file version is not supported");
			return false;
		}

		readSize = fread(&mtc->nrOfVertices, 1, sizeof(int), mtcFile);
		readSize = fread(&mtc->nrOfVolumes, 1, sizeof(int), mtcFile);

		char *vtcFileName = new char[256];
		char endSign = '1';
		int counter = 0;
		while ( endSign != '\0') {
			readSize = fread(&endSign, 1, sizeof(unsigned char), mtcFile);
			vtcFileName[counter] = endSign;
			counter++ ;
		}
		vtcFileName[counter] = '\0';
		mtc->vtcFileName = vtcFileName;

		char *PRTFileName = new char[256];
		char endSign2 = '1';
		counter = 0;
		while ( endSign2 != '\0') {
			readSize = fread(&endSign2, 1, sizeof(unsigned char), mtcFile);
			PRTFileName[counter] = endSign2;
			counter++ ;
		}
		PRTFileName[counter] = '\0';
		mtc->prtFileName = PRTFileName;

		readSize = fread(&mtc->hemoDynDelay, 1, sizeof(int), mtcFile);
		readSize = fread(&mtc->TR, 1, sizeof(float), mtcFile);
		readSize = fread(&mtc->HRFdelta, 1, sizeof(float), mtcFile);
		readSize = fread(&mtc->HRFtau, 1, sizeof(float), mtcFile);
		readSize = fread(&mtc->segmentSize, 1, sizeof(int), mtcFile);
		readSize = fread(&mtc->segmentOffset, 1, sizeof(int), mtcFile);
		readSize = fread(&mtc->MTCDataType, 1, sizeof(char), mtcFile);

		int dataSize = (mtc->nrOfVertices * mtc->nrOfVolumes);
		if (readData) {
			float *data = new float[dataSize];
			readSize += fread(data, 1, (dataSize * sizeof(float)), mtcFile);
			mtc->data = data;
		}
		fclose(mtcFile);

	} catch(...) {
		qxLogText("Error while reading *.mtc file.");
	}
	return mtc;
}

smpStruct *getsmp(char *filename, bool exportfromdisk) {

	bool success = true;
	smpStruct *smp = new smpStruct;
	if (exportfromdisk) {
		success = readsmp(filename, smp);
	} else {
		SMPs_Header *bvqxsmpshdr = new SMPs_Header;
		success = qxGetSMPsOfCurrentSRF(bvqxsmpshdr);
		// now convert BV structure to superstructure
		if (success){
			smp = convertSMP(bvqxsmpshdr);
		//	smp->versionNr = 3.5f;
		}
	}
	if (!success){
		smp->versionNr = -1;
		qxLogText("Reading of surface map did not succeed.");
	}
	return smp;
}


bool readsmp(char *smpFileName, smpStruct *smp) {

	FILE *smpFile;
	int readSize;
	bool readData = true;
	char *txt = new char[1024];
	smp->origName = smpFileName;

	try {

		smpFile = fopen(smpFileName,"rb");
		if (smpFile == NULL) {
			qxMessageBox("File could not be accessed.\nPlease check if it is opened in other programs.");
		}

		readSize = fread(&smp->versionNr, 1, sizeof(short int), smpFile);
		readSize += fread(&smp->nrOfVertices, 1, sizeof(int), smpFile);
		readSize += fread(&smp->nrOfMaps, 1, sizeof(short int), smpFile);

		char *SRFFileName = new char[256];
		char endSign = '1';
		int counter = 0;
		while ( endSign != '\0') {
			readSize += fread(&endSign, 1, sizeof(unsigned char), smpFile);
			SRFFileName[counter] = endSign;
			counter++ ;
		}
		SRFFileName[counter] = '\0';
		smp->srfFileName = SRFFileName;
		int i;
		srfStatMap *srfStatMaps = new srfStatMap[smp->nrOfMaps];

		int smpDataSize = smp->nrOfVertices * sizeof(float);
		for (i=0; i < smp->nrOfMaps; i++) {

			readSize += fread(&srfStatMaps[i].mapType, 1, sizeof(int), smpFile);
			if (srfStatMaps[i].mapType == 3) {
				// crosscorrelation
				readSize += fread(&srfStatMaps[i].nrOfLags, 1, sizeof(int), smpFile);
				readSize += fread(&srfStatMaps[i].minLag, 1, sizeof(int), smpFile);
				readSize += fread(&srfStatMaps[i].maxLag, 1, sizeof(int), smpFile);
				readSize += fread(&srfStatMaps[i].ccOverlay, 1, sizeof(int), smpFile);
			}

			readSize += fread(&srfStatMaps[i].clusterSize, 1, sizeof(int), smpFile);
			readSize += fread(&srfStatMaps[i].enableClusterCheck, 1, sizeof(char), smpFile);
			readSize += fread(&srfStatMaps[i].critValThresh, 1, sizeof(float), smpFile);
			readSize += fread(&srfStatMaps[i].maxThresh, 1, sizeof(float), smpFile);

			if (smp->versionNr > 3) readSize += fread(&srfStatMaps[i].contrXrcmaps, 1, sizeof(int), smpFile);
			readSize += fread(&srfStatMaps[i].df1, 1, sizeof(int), smpFile);
			readSize += fread(&srfStatMaps[i].df2, 1, sizeof(int), smpFile);
			if (smp->versionNr > 4) readSize += fread(&srfStatMaps[i].showPosNegFlag, 1, sizeof(int), smpFile);
			readSize += fread(&srfStatMaps[i].bonfCorr, 1, sizeof(int), smpFile);
			readSize += fread(&srfStatMaps[i].RGBLowerThreshPos, 1, sizeof(char)*3, smpFile);
			readSize += fread(&srfStatMaps[i].RGBUpperThreshPos, 1, sizeof(char)*3, smpFile);
			if (smp->versionNr > 3) readSize += fread(&srfStatMaps[i].RGBLowerThreshNeg, 1, sizeof(char)*3, smpFile);
			if (smp->versionNr > 3) readSize += fread(&srfStatMaps[i].RGBUpperThreshNeg, 1, sizeof(char)*3, smpFile);
			readSize += fread(&srfStatMaps[i].enableSMPcolor, 1, sizeof(char), smpFile);

			if (smp->versionNr > 4) {
				char *lutName = new char[256];
				char endSign = '1';
				int counter = 0;
				while ( endSign != '\0') {
					readSize += fread(&endSign, 1, sizeof(unsigned char), smpFile);
					lutName[counter] = endSign;
					counter++ ;
				}
				lutName[counter] = '\0';
				srfStatMaps[i].LUTName = lutName; //escapize_string(surfaceMapName);
			}
			readSize += fread(&srfStatMaps[i].transparentColorFactor, 1, sizeof(float), smpFile);

			char *surfaceMapName = new char[256];
			char endSign = '1';
			int counter = 0;
			while ( endSign != '\0') {
				readSize += fread(&endSign, 1, sizeof(unsigned char), smpFile);
				surfaceMapName[counter] = endSign;
				counter++ ;
			}
			surfaceMapName[counter] = '\0';
			srfStatMaps[i].surfaceMapName = surfaceMapName; //escapize_string(surfaceMapName);

			if (readData) {
				float *smpData = new float[smpDataSize];
				try {
					readSize += fread(smpData, 1, smpDataSize, smpFile);
				} catch (...) {
					qxLogText("Error while reading SMP data");
					return false;
				}
				srfStatMaps[i].mapData = smpData;
			} else {
				fseek(smpFile, smpDataSize, SEEK_CUR);
			}
		}
		smp->srfStatMaps = srfStatMaps;
		fclose(smpFile);

	} catch (...) {
		printf("Error while reading SMP file");
	}

	return smp;
}

poiStruct *getpoi(char *filename, bool exportfromdisk) {
	
	bool success = true;
	poiStruct *poi = new poiStruct;
	if (exportfromdisk) {
		success = readpoi(filename, poi);
	} else {
		POIs_Header *bvqxpoishdr = new POIs_Header;
		success = qxGetPOIs(bvqxpoishdr);
		SRF_Header *srfh = new SRF_Header;
		success = (bool)qxGetCurrentSRF(srfh);
		// now convert BV structure to superstructure
		if (success){
			poi = convertPOI(bvqxpoishdr, srfh->NrOfVertices); // veranderen naar: POIs_Header *bvqxpois, int NrOfMeshVertices
		}
	}
	if (!success){
		poi->FileVersion = -1;
		qxLogText("Reading of patch of interest did not succeed.");
	}
	return poi;
}



bool readpoi(char *poiFileName, poiStruct *poi) {
	
	bool success = true;
	char tmp[100];
	char poiname[100];
	std::string line;

	// lees header
	try{
		poi->POIsFileName = poiFileName;
		std::ifstream poiFile(poiFileName, std::ifstream::in);
		poiFile >> tmp >> poi->FileVersion; // not backwards compatible!
		char *fromMeshFile = new char[256];
		poiFile >> tmp >> fromMeshFile;
		poi->FromMeshFile = fromMeshFile;
		poiFile >> tmp >> poi->NrOfMeshVertices;
		poiFile >> tmp >> poi->NrOfPOIs;
		
		// loop over data
		poiSubStruct *patch = new poiSubStruct[poi->NrOfPOIs];
		int red, green, blue;
		int startval = 0;//1056964608;

		for (int i = 0; i<poi->NrOfPOIs; i++){
			
			std::getline(poiFile, line);
			while (line == "") std::getline(poiFile, line);
			std::string::size_type first_quote = line.find_first_of('"');
			std::string name = line.substr(first_quote+1, line.length()-first_quote-2);
			char *poiname = new char[256];
			sprintf(poiname, "%s", name.c_str());
			patch[i].POIName = poiname;
						
			std::getline(poiFile, line);
			first_quote = line.find_first_of('"');
			std::string textfilename = line.substr(first_quote+1, line.length()-first_quote-2);
			char *infofilename = new char[256];
			sprintf(infofilename, "%s", textfilename.c_str());
			patch[i].InfoFileName = infofilename;
			
			poiFile >> tmp >> red >> green >> blue;
			patch[i].ColorOfPOI = startval + red * pow(16.0f,4) + green * pow(16.0f,2) + blue;
			
			poiFile >> tmp >> patch[i].ReferenceVertex; 
			poiFile >> tmp >> patch[i].NrOfVertices; 

			
			int *data = new int[patch[i].NrOfVertices];
			for (int j = 0; j< patch[i].NrOfVertices; j++){
				poiFile >> data[j];
			}
			patch[i].POIdata = data;
		}
		poi->poi = patch;
		poiFile >> tmp >> poi->NrOfPOIMTCs;
		poiFile.close();
	} catch (...) {
		qxLogText("Error while reading *.poi file.");
		success = false;
	}
	return success;
}


bool writesrf(srfStruct *srf) {

	bool success;

	try {

		int writeSize;
		FILE *srfFile;
		char srfExt[] = ".srf";
		char *srfFileName = exchangeExtension(srf->origName, srfExt);
		int nrVertices = srf->NrOfVertices;
		int totVertSz = nrVertices * sizeof(float);



		srfFile = fopen(srfFileName, "wb");
		writeSize = fwrite(&srf->fileVersion, 1, sizeof(float), srfFile);
		writeSize = fwrite(&srf->reserved, 1, sizeof(int), srfFile);
		writeSize = fwrite(&nrVertices, 1, sizeof(int), srfFile);
		writeSize = fwrite(&srf->NrOfTriangles, 1, sizeof(int), srfFile);
		writeSize = fwrite(&srf->MeshCenterX, 1, sizeof(float), srfFile);
		writeSize = fwrite(&srf->MeshCenterY, 1, sizeof(float), srfFile);
		writeSize = fwrite(&srf->MeshCenterZ, 1, sizeof(float), srfFile);
		writeSize = fwrite(srf->VertexX, 1, totVertSz, srfFile);
		writeSize = fwrite(srf->VertexY, 1, totVertSz, srfFile);
		writeSize = fwrite(srf->VertexZ, 1, totVertSz, srfFile);
		writeSize = fwrite(srf->NormalX, 1, totVertSz, srfFile);
		writeSize = fwrite(srf->NormalY, 1, totVertSz, srfFile);
		writeSize = fwrite(srf->NormalZ, 1, totVertSz, srfFile);
		writeSize = fwrite(&srf->ConvCurvRed, 1, sizeof(float), srfFile);
		writeSize = fwrite(&srf->ConvCurvGreen, 1, sizeof(float), srfFile);
		writeSize = fwrite(&srf->ConvCurvBlue, 1, sizeof(float), srfFile);
		writeSize = fwrite(&srf->ConvCurvAlpha, 1, sizeof(float), srfFile);
		writeSize = fwrite(&srf->ConcCurvRed, 1, sizeof(float), srfFile);
		writeSize = fwrite(&srf->ConcCurvGreen, 1, sizeof(float), srfFile);
		writeSize = fwrite(&srf->ConcCurvBlue, 1, sizeof(float), srfFile);
		writeSize = fwrite(&srf->ConcCurvAlpha, 1, sizeof(float), srfFile);
		writeSize = fwrite(srf->MeshColor, 1, (nrVertices * sizeof(int)), srfFile);

		int vertex;
		for (vertex = 0; vertex < nrVertices; vertex++) {
			writeSize = fwrite(&srf->neighb[vertex].nrOfNeighbours, 1, sizeof(int), srfFile);
			writeSize = fwrite(srf->neighb[vertex].neighbourLst, 1, (srf->neighb[vertex].nrOfNeighbours * sizeof(int)), srfFile);
		}
		writeSize = fwrite(srf->triangleSequence, 1, (3 * srf->NrOfTriangles * sizeof(int)), srfFile);
		writeSize = fwrite(&srf->nrOfTriangleStripElements, 1, sizeof(int), srfFile);
		if (srf->nrOfTriangleStripElements > 0) {
			writeSize = fwrite(&srf->stripElemSequence, 1, (srf->nrOfTriangleStripElements * sizeof(int)), srfFile);
		}
		fclose(srfFile);
	} catch (...) {
		printf("Error while writing SRF file");
		success = false;
	}

	return success;
}

bool writemtc(mtcStruct *mtc) {

	bool success;

	try {

		int writeSize;
		FILE *mtcFile;
		char mtcExt[] = ".mtc";
		char *mtcFileName = exchangeExtension(mtc->origName, mtcExt);

		mtcFile = fopen(mtcFileName, "wb");
		writeSize = fwrite(&mtc->fileVersion, 1, sizeof(int), mtcFile);
		writeSize += fwrite(&mtc->nrOfVertices, 1, sizeof(int), mtcFile);
		writeSize += fwrite(&mtc->nrOfVolumes, 1, sizeof(int), mtcFile);
		writeSize += fwrite(mtc->vtcFileName, 1, (strlen(mtc->vtcFileName)+1), mtcFile);
		writeSize += fwrite(mtc->prtFileName, 1, (strlen(mtc->prtFileName)+1), mtcFile);
		writeSize += fwrite(&mtc->hemoDynDelay, 1, sizeof(int), mtcFile);
		writeSize += fwrite(&mtc->TR, 1, sizeof(float), mtcFile);
		writeSize += fwrite(&mtc->HRFdelta, 1, sizeof(float), mtcFile);
		writeSize += fwrite(&mtc->HRFtau, 1, sizeof(float), mtcFile);
		writeSize += fwrite(&mtc->segmentSize, 1, sizeof(int), mtcFile);
		writeSize += fwrite(&mtc->segmentOffset, 1, sizeof(int), mtcFile);
		writeSize += fwrite(&mtc->MTCDataType, 1, sizeof(char), mtcFile);
		writeSize += fwrite(mtc->data, 1, (sizeof(float) * mtc->nrOfVertices * mtc->nrOfVolumes), mtcFile);
		fclose(mtcFile);

	} catch (...) {
		printf("Error while saving MTC file");
		success = false;
	}

	return success;
}

bool writesmp(smpStruct *smp) {

	bool success;

	try {

		int writeSize;
		FILE *smpFile;
		char smpExt[] = ".smp";
		char *smpFileName = exchangeExtension(smp->origName, smpExt);
		short int nrOfMaps = smp->nrOfMaps;

		smpFile = fopen(smpFileName, "wb");
		writeSize = fwrite(&smp->versionNr, 1, sizeof(short int), smpFile);
		writeSize = fwrite(&smp->nrOfVertices, 1, sizeof(int), smpFile);
		writeSize = fwrite(&nrOfMaps, 1, sizeof(short int), smpFile);
		writeSize = fwrite(smp->srfFileName, 1, (strlen(smp->srfFileName)+1), smpFile); // added 07-01-11

		int map;
		for (map=0; map < nrOfMaps; map++) {

			int mapType = smp->srfStatMaps[map].mapType;
			writeSize = fwrite(&mapType, 1, sizeof(int), smpFile);
			if (mapType == 3) {
				writeSize = fwrite(&smp->srfStatMaps[map].nrOfLags, 1, sizeof(int), smpFile);
				writeSize = fwrite(&smp->srfStatMaps[map].minLag, 1, sizeof(int), smpFile);
				writeSize = fwrite(&smp->srfStatMaps[map].maxLag, 1, sizeof(int), smpFile);
				writeSize = fwrite(&smp->srfStatMaps[map].ccOverlay, 1, sizeof(int), smpFile);
			}
			writeSize = fwrite(&smp->srfStatMaps[map].clusterSize, 1, sizeof(int), smpFile);
			writeSize = fwrite(&smp->srfStatMaps[map].enableClusterCheck, 1, sizeof(char), smpFile);
			writeSize = fwrite(&smp->srfStatMaps[map].critValThresh, 1, sizeof(float), smpFile);
			writeSize = fwrite(&smp->srfStatMaps[map].maxThresh, 1, sizeof(float), smpFile);
			writeSize = fwrite(&smp->srfStatMaps[map].useValuesAboveThreshold, 1, sizeof(int), smpFile); // added 07-01-11
			if (mapType == 3) {
				writeSize = fwrite(&smp->srfStatMaps[map].contrXrcmaps, 1, sizeof(int), smpFile);
			}
			writeSize = fwrite(&smp->srfStatMaps[map].df1, 1, sizeof(int), smpFile);
			writeSize = fwrite(&smp->srfStatMaps[map].df2, 1, sizeof(int), smpFile); // used? (only for maptype 1?)
			writeSize = fwrite(&smp->srfStatMaps[map].showPosNegFlag, 1, sizeof(int), smpFile); // added 07-01-11
			writeSize = fwrite(&smp->srfStatMaps[map].bonfCorr, 1, sizeof(int), smpFile);
			writeSize = fwrite(&smp->srfStatMaps[map].RGBLowerThreshPos, 1, (3*sizeof(char)), smpFile);
			writeSize = fwrite(&smp->srfStatMaps[map].RGBUpperThreshPos, 1, (3*sizeof(char)), smpFile);
			writeSize = fwrite(&smp->srfStatMaps[map].RGBLowerThreshNeg, 1, (3*sizeof(char)), smpFile);// added 07-01-11
			writeSize = fwrite(&smp->srfStatMaps[map].RGBUpperThreshNeg, 1, (3*sizeof(char)), smpFile);// added 07-01-11
			writeSize = fwrite(&smp->srfStatMaps[map].enableSMPcolor, 1, sizeof(char), smpFile);
			writeSize = fwrite(smp->srfStatMaps[map].LUTName, 1, (strlen(smp->srfStatMaps[map].LUTName)+1), smpFile); // added 07-01-11
			writeSize = fwrite(&smp->srfStatMaps[map].transparentColorFactor, 1, sizeof(float), smpFile);
			writeSize = fwrite(smp->srfStatMaps[map].surfaceMapName, 1, (strlen(smp->srfStatMaps[map].surfaceMapName)+1), smpFile);
			writeSize = fwrite(smp->srfStatMaps[map].mapData, 1, (smp->nrOfVertices * sizeof(float)), smpFile);
		}
		fclose(smpFile);
		
		/*
		 @FileVersion = 5;
		 @NrOfVertices = 40962;
		 @NrOfMaps = 1;
		 @NameOfOriginalSRF = 'untitled.srf';
		 @Map.Type = 1;
		 @Map.NrOfLags = [];
		 @Map.MinLag = [];
		 @Map.MaxLag = [];
		 @Map.CCOverlay = [];
		 @Map.ClusterSize = 25;
		 @Map.EnableClusterCheck = 0;
		 @Map.LowerThreshold = 2.5;
		 @Map.UpperThreshold = 8;
		 @Map.UseValuesAboveThresh = 1;
		 @Map.DF1 = 249;
		 @Map.DF2 = 0;
		 @Map.ShowPositiveNegativeFlag = 3;
		 @Map.BonferroniValue = @NrOfVertices;
		 @Map.RGBLowerThreshPos = [192,   0,   0];
		 @Map.RGBUpperThreshPos = [255, 192,   0];
		 @Map.RGBLowerThreshNeg = [  0,   0, 128];
		 @Map.RGBUpperThreshNeg = [  0, 128, 255];
		 @Map.UseRGBColor = 1;
		 @Map.LUTName = '<default>';
		 @Map.TransColorFactor = 1;
		 @Map.Name = 'New Map';
		 @Map.SMPData = zeros(@NrOfVertices, 1);
		 */
		
		
		
	} catch (...) {
		printf("Error while writing SMP file");
		success = false;
	}
	return success;
}


bool writepoi(poiStruct *poi) {

	bool success = true;
	std::ofstream poifile(poi->POIsFileName);
	int color, red, green, blue;
	int startval = 0;//1056964608;
	if (poifile.bad()) {
		printf("Error while writing POI file");
		success = false;
	} else {
		
		poifile << "FileVersion:      " << poi->FileVersion << "\n\n";
		poifile << "FromMeshFile:     \"" << poi->FromMeshFile << "\"\n\n";
		poifile << "NrOfMeshVertices: " << poi->NrOfMeshVertices << "\n\n\n";
		poifile << "NrOfPOIs:         " << poi->NrOfPOIs << "\n\n";
		
		for (int i=0; i<poi->NrOfPOIs; i++) {
			
			poifile << "NameOfPOI:        \"" << poi->poi[i].POIName << "\"\n";
			poifile << "InfoTextFile:     \"" << poi->poi[i].InfoFileName << "\"\n";
			color = poi->poi[i].ColorOfPOI;
			color -= startval;
			red = color/pow(16.0f,4);
			green = (color-red*pow(16.0f,4))/pow(16.0f,2);
			blue = color-red*pow(16.0f,4)-green*pow(16.0f,2);
			poifile << "ColorOfPOI:       " << red << " " << green << " " << blue << "\n";			
			poifile << "LabelVertex:      " << poi->poi[i].ReferenceVertex << "\n"; 
			poifile << "NrOfVertices:     " << poi->poi[i].NrOfVertices << "\n";
			for (int j=0; j<poi->poi[i].NrOfVertices; j++) {
				poifile << poi->poi[i].POIdata[j] << "\n";
			}
			poifile << "\n";
		}
	} 
	poifile << "\n" << "NrOfPOIMTCs: " << poi->NrOfPOIMTCs << "\n";
	poifile.close();
	
	return success;
}


