/*
 *  classify.cpp
 *  gifti
 *
 *  Created by Hester Breman and Joost Mulders on 22/06/2008.
 *
 *  Copyright 2008 Brain Innovation B.V. All rights reserved.
 
 13-09-11: added NIFTI_INTENT_RETINOTOPY_/BVQX_MAPTYPE_RETINOTOPY_ types in GetGIFTIIntent() and getBVQXmaptype()
 
 *
 */
#include <cstdlib>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string> 
#include <string.h>
#include <iostream>
#include <map> 
#include <iterator>
#include <errno.h>
using namespace std;

#include "classify.h"
#include "constants.h"
#include "nifti1.h"
#include "gifti_io.h"
#include "BVQXPluginInterfaceExtern.h"


char *getAnatomicalStructurePrimary(char *origsrfname) {

	bool found = false;
	string srffilename = origsrfname;
	char *anatStructPrimary = new char[256];
	string currentPattern;
	string::size_type idx; // or std::string::size_type idx;
	map<string, string> anatStructPrimaryMap;
	map<string, string>::iterator iter; 
	anatStructPrimaryMap["_RH"] = "CortexRight";
	anatStructPrimaryMap["_LH"] = "CortexLeft";
	anatStructPrimaryMap["_RHLH"] = "CortexRightAndLeft";	
	anatStructPrimaryMap["_HEAD"] = "Head";		
	
	for (iter = anatStructPrimaryMap.begin(); iter != anatStructPrimaryMap.end(); ++iter) {
	
		currentPattern = (*iter).first;
		idx = srffilename.find(currentPattern);
		if (idx == string::npos) {
			sprintf(anatStructPrimary,  ((*iter).second).c_str());
			break;
		}
//		sprintf(menustr+strlen(menustr), "%d %c %s<BR>\n\n", (*iter).first, '=', ((*iter).second).c_str()); 				
	}
		
	if (!found) {
		sprintf(anatStructPrimary, "Unknown");
	}	
/*

"CortexRight" 
"CortexRightAndLeft"
"Cerebellum" 
"Head"
"HippocampusLeft" 
"HippocampusRight"
*/


	return anatStructPrimary;
}

char *getAnatomicalStructureSecondary(char *origsrfname) {

	bool found = false;
	string srffilename = origsrfname;
	char *anatStructSecondary = new char[256];
	string currentPattern;
	string::size_type idx; // or std::string::size_type idx;
	map<string, string> anatStructSecondaryMap;
	map<string, string>::iterator iter; 
	
	anatStructSecondaryMap["_WM"] = "GrayWhite"; //  The white/gray boundary.
	anatStructSecondaryMap["_GM"] = "Pial"; // The pial (gray/CSF) boundary.
//	anatStructSecondaryMap["_RHLH"] = "MidThickness";	//
	
	for (iter = anatStructSecondaryMap.begin(); iter != anatStructSecondaryMap.end(); ++iter) {
	
		currentPattern = (*iter).first;
		idx = srffilename.find(currentPattern);
		if (idx == string::npos) {
			sprintf(anatStructSecondary,  ((*iter).second).c_str());
			found = true;
			break;
		}
//		sprintf(menustr+strlen(menustr), "%d %c %s<BR>\n\n", (*iter).first, '=', ((*iter).second).c_str()); 				
	}
	if (!found) {
		sprintf(anatStructSecondary, "Unknown");
	}	

	return anatStructSecondary;
}

char *getGeometricType(char *origsrfname) {

/* Reconstruction The surface as it appears
immediately after reconstruction
with a “blocky” appearance.
Anatomical Represents an anatomical structure
such as the white/gray boundary.
Inflated A surface that has been inflated.
VeryInflated A surface that has been highly
inflated.
Spherical A spherical surface.
SemiSpherical A sphere with one-half flattened.
Ellipsoid An ellipsoidal surface.
Flat A flattened surface.
Hull A hull surface (wrapping around a
surface, ie: cortex with sulci filled.*/

	bool found = false;
	string srffilename = origsrfname;
	char *geometricType = new char[256];
	string currentPattern;
	string::size_type idx; // or std::string::size_type idx;
	map<string, string> geometricTypeMap;
	map<string, string>::iterator iter; 
	geometricTypeMap["_RECOSM"] = "Reconstruction";
	geometricTypeMap["_RECO"] = "Reconstruction";
	geometricTypeMap["_SPH"] = "Sphere";
	geometricTypeMap["_SPHERE"] = "Sphere";
	geometricTypeMap["_INFL1500"] = "VeryInflated";
	geometricTypeMap["_INFL1000"] = "Inflated";
	geometricTypeMap["_INFL500"] = "Inflated";
	geometricTypeMap["_INFL_"] = "Inflated";
	geometricTypeMap["_FLAT"] = "Flat";
					
	for (iter = geometricTypeMap.begin(); iter != geometricTypeMap.end(); ++iter) {
	
		currentPattern = (*iter).first;
		idx = srffilename.find(currentPattern);
		if (idx == string::npos) {
			sprintf(geometricType,  ((*iter).second).c_str());
			break;
		}
//		sprintf(menustr+strlen(menustr), "%d %c %s<BR>\n\n", (*iter).first, '=', ((*iter).second).c_str()); 				
	}
		
	if (!found) {
		sprintf(geometricType, "Unknown");
	}	
	return geometricType;
}

char *getTopologicalType(char *origsrfname) {

	bool found = false;
	string srffilename = origsrfname;
	char *topologicalType = new char[256];
	string currentPattern;
	string::size_type idx; // or std::string::size_type idx;
	map<string, string> topologicalTypeMap;
	map<string, string>::iterator iter; 
	topologicalTypeMap["_FLAT"] = "FLAT";

	for (iter = topologicalTypeMap.begin(); iter != topologicalTypeMap.end(); ++iter) {

		currentPattern = (*iter).first;
		idx = srffilename.find(currentPattern);
		if (idx == string::npos) {
			sprintf(topologicalType,  ((*iter).second).c_str());
			break;
		}
		//		sprintf(menustr+strlen(menustr), "%d %c %s<BR>\n\n", (*iter).first, '=', ((*iter).second).c_str()); 				
	}

	if (!found) {
		sprintf(topologicalType, "CLOSED");
	}	
	return topologicalType;

}

int getGIfTIIntent(int bvqxmaptype) {

	switch(bvqxmaptype){

		case BVQX_MAPTYPE_T_STAT:					return NIFTI_INTENT_TTEST;
		case BVQX_MAPTYPE_CORRELATION:				return NIFTI_INTENT_CORREL;
		case BVQX_MAPTYPE_CROSSCORRELATION:			return NIFTI_INTENT_CORREL;
		case BVQX_MAPTYPE_F_STAT:					return NIFTI_INTENT_FTEST;
		case BVQX_MAPTYPE_Z_STAT:					return NIFTI_INTENT_ZSCORE;
		case BVQX_MAPTYPE_BETA:						return NIFTI_INTENT_BETA;
		case BVQX_MAPTYPE_CHI2:						return NIFTI_INTENT_CHISQ;
		case BVQX_MAPTYPE_THICKNESS:			    return NIFTI_INTENT_SHAPE;
		case BVQX_MAPTYPE_RETINOTOPY_POLAR:			return NIFTI_INTENT_RETINOTOPY_POLAR;
		case BVQX_MAPTYPE_RETINOTOPY_ECCENTRICITY:	return NIFTI_INTENT_RETINOTOPY_ECCENTRICITY;
		case BVQX_MAPTYPE_RETINOTOPY_FIELD_SIGN:	return NIFTI_INTENT_RETINOTOPY_FIELD_SIGN;
		default:									return NIFTI_INTENT_NONE;
	}
	return NIFTI_INTENT_NONE; // here one should never arrive	
}

int getBVQXmaptype(int niftimaptype) {

	switch(niftimaptype){

		case NIFTI_INTENT_TTEST:					return BVQX_MAPTYPE_T_STAT;
		case NIFTI_INTENT_CORREL:					return BVQX_MAPTYPE_CORRELATION;
		case NIFTI_INTENT_FTEST:					return BVQX_MAPTYPE_F_STAT;
		case NIFTI_INTENT_ZSCORE:					return BVQX_MAPTYPE_Z_STAT;
		case NIFTI_INTENT_BETA:						return BVQX_MAPTYPE_BETA;
		case NIFTI_INTENT_CHISQ:					return BVQX_MAPTYPE_CHI2;
		case NIFTI_INTENT_SHAPE:				    
			if (qxYesNoMessageBox("GIFTI type is 'Shape'\n\nIs this a Cortical Thickness (Yes) or Curvature (No) map"))
				return BVQX_MAPTYPE_THICKNESS;
			else
				return BVQX_MAPTYPE_CURVATURE; //== T_STAT type is used to store curvature information
		case NIFTI_INTENT_RETINOTOPY_POLAR:			return BVQX_MAPTYPE_RETINOTOPY_POLAR;
		case NIFTI_INTENT_RETINOTOPY_ECCENTRICITY:	return BVQX_MAPTYPE_RETINOTOPY_ECCENTRICITY;
		case NIFTI_INTENT_RETINOTOPY_FIELD_SIGN:	return BVQX_MAPTYPE_RETINOTOPY_FIELD_SIGN;			
		default:									return BVQX_MAPTYPE_T_STAT;
	}
	return BVQX_MAPTYPE_T_STAT; // here one should never arrive	
}


char *getBVMapName(gifti_image *gii, int mapnr){

	char *mapname = new char[256];
	char *txt = new char[256];

	for (int valnr = 0; valnr < gii->darray[mapnr]->meta.length; valnr++) { // loop over CDATA tags
		char *name = gii->darray[mapnr]->meta.name[valnr];
		if (strcmp(name, "Name") == 0 || strcmp(name, "name") == 0) {
			sprintf(mapname, "%s", gii->darray[mapnr]->meta.value[valnr]);
			sprintf(txt, "Found Name: '%s' for Map: %i", mapname, mapnr);
			qxLogText(txt);
		}
	}

	if (mapname == NULL) //if mapname has not be found in meta data of gifti DA
		sprintf(mapname, "%s", nifti_intent_string(gii->darray[mapnr]->intent));

	return mapname;
}

bool iscurvature(char *bvqxmapname) {

	bool iscurv = false;
	if (strcmp(bvqxmapname, "Curvature") == 0)	iscurv = true;
	return iscurv;
}

bool getdistributionparams(gifti_image *gii, int mapnr, int bvqxmaptype, float *distrparams) {
	
	bool success = true;
	char *txt = new char[256];
//	float *distrparams = new float[3];
	distrparams[0] = 0.0; distrparams[1] = 0.0; distrparams[2]=  0.0;
	
	/* Intent_code, intent_p1, intent_p2, intent_p3 – Included in Functional DataArray metadata 
	 and contains a NIFTI intent code and parameters describing the type of statistical test.
	 page 15 GIFTI data format description */
	
	bool intentparam1set = false;
	int valnr;
	for (valnr = 0; valnr < gii->darray[mapnr]->meta.length; valnr++) { // loop over CDATA tags
			
				//if (errno != ERANGE) {
		char *name = gii->darray[mapnr]->meta.name[valnr];
		if (strcmp(name, "intent_p1") == 0) {// nou crasht ie hier (logisch, hierboven stond 'value' ipv 'name'
			distrparams[0] = (float)atof(gii->darray[mapnr]->meta.value[valnr]);
			intentparam1set = true;							 
			sprintf(txt, "Found distribution parameter 1: '%f' of intent '%s'", distrparams[0], nifti_intent_string(gii->darray[mapnr]->intent));
			qxLogText(txt);
		}
		if (strcmp(name, "intent_p2") == 0) distrparams[1] = (float)atof(gii->darray[mapnr]->meta.value[valnr]);
		if (strcmp(name, "intent_p3") == 0) distrparams[2] = (float)atof(gii->darray[mapnr]->meta.value[valnr]);
	}
		
	if (!intentparam1set && getnrofdistributionparams(bvqxmaptype) > 0) {
		
		// als niet gezet, kijk naar soort verdeling
		// als verdeling geen parameters nodig heeft, zet alles op 0
		if (gii->darray[mapnr]->intent != NIFTI_INTENT_ZSCORE || 
			gii->darray[mapnr]->intent != NIFTI_INTENT_PVAL) {

			float defval = 0.0; float minval = -1000.0; float maxval = 1000.0; float usrval = 0.0;						
			usrval = qxGetFloatInput("Please enter the first degrees of freedom parameter (see manual for help)", defval, minval, maxval);
			if (usrval == (minval-1.0)) {
				success = false;
			} else {
				distrparams[0] = usrval;
				// kijk of tweede waarde nodig is:
				if (getnrofdistributionparams(bvqxmaptype) > 1) {
					usrval = qxGetFloatInput("Please enter the second degrees of freedom parameter ", defval, minval, maxval);
					if (usrval == (minval-1.0)) {
						success = false; // willen we dit?
					} else {
						distrparams[1] = usrval;
					}
				} // end 2nd dof
			} // end 1st dof 
		} // end intent != zscore || pval
	} // end intent is not present

	return success; // 16-12-10: changed from 'distributionparams' to success
}

int getnrofdistributionparams(int bvqxmaptype){

	switch(bvqxmaptype) {
		case BVQX_MAPTYPE_T_STAT:			return 1;
		case BVQX_MAPTYPE_CORRELATION:		return 1;
		case BVQX_MAPTYPE_CROSSCORRELATION: return 1; // uitzoeken!!!
		case BVQX_MAPTYPE_F_STAT:			return 2;
		case BVQX_MAPTYPE_Z_STAT:			return 0;
		case BVQX_MAPTYPE_PSC:				return 0; 
		case BVQX_MAPTYPE_CHI2:				return 1;
		case BVQX_MAPTYPE_BETA:				return 2;
		case BVQX_MAPTYPE_THICKNESS:		return 0;
		//case BVQX_MAPTYPE_CURVATURE:		return 0; //CURVATURE == T_STAT dus komt hier nooit
		default: return 0;
	}
	

}