
/* 
 13-09-11: added facilitation for retinotopy data in export_smp()
 */

// plugin specific headers
#include "SRF_Header.h"
#include "Plugin_SRF_Header.h"
#include "BVQXPluginInterfaceExtern.h"
#include "MTC_Header.h"
#include "SMP_Header.h"
#include "POI_Header.h"
#include "export.h"
#include "bvqx_io.h"
#include "gifti_io.h"
#include "nifti1.h"
#include "classify.h"
#include "convertbvqxstructs.h"
#include "info.h"
#include "constants.h"
#include "surfacefunctions.h"


// export BrainVoyager to GIfTI


bool export_brainvoyager(int filetypeid, bool useCurrentFile, bool exportToBinary, bool doExportColors) {

        int usrresp;
    	bool success = true;
		bool exportfromdisk = (!useCurrentFile);
   		char *filename = new char[501];
		SRF_Header *bvqxsrfhdr = new SRF_Header;
        bool askexport = (bool)qxGetCurrentSRF(bvqxsrfhdr);
        if(!exportfromdisk) sprintf(filename, "none");
			switch (filetypeid){
				case TypeSrf:
					if (exportfromdisk) {
						usrresp = qxGetOpenFileName("Please select the BrainVoyager surface file", "", "*.srf", filename);
						if (usrresp < 1) {
							qxMessageBox("No surface file was selected. Procedure finished.");
							return false;
						}

					}
					if (!export_srf(filename, exportfromdisk, exportToBinary, doExportColors)) success = false;
					break;
				case TypeMtc:
					if (exportfromdisk) {
						usrresp = qxGetOpenFileName("Please select the BrainVoyager mesh time course file", "", "*.mtc", filename);
						if (usrresp < 1) {
							qxMessageBox("No mesh time course file was selected. Procedure finished.");
							return false;
						}
					}
					if (!export_mtc(filename, exportfromdisk, exportToBinary)) success = false;
					break;
				case TypeSmp:
					if (exportfromdisk) {
						usrresp = qxGetOpenFileName("Please select the BrainVoyager surface map file", "", "*.smp", filename);
						if (usrresp < 1) {
							qxMessageBox("No surface map file was selected. Procedure finished.");
							return false;
						}
					}
					if (!export_smp(filename, exportfromdisk, exportToBinary)) success = false;
					break;
				case TypePoi:
					if (exportfromdisk) {
						usrresp = qxGetOpenFileName("Please select the BrainVoyager patch of interest file", "", "*.poi", filename);
						if (usrresp < 1) {
							qxMessageBox("No *.poi file was selected. Procedure finished.");
							return false;
						}					
					}
					if (!export_poi(filename, exportfromdisk, exportToBinary)) success = false;
					break;
			} // end submenu switch
        return success;
}

bool export_srf(char *filename, bool exportfromdisk, bool exportToBinary, bool doExportColors) {

	bool success = true;
	srfStruct *srf = getsrf(filename, exportfromdisk);
	if (srf->fileVersion != -1.0) {
		headerdump_srf(srf, false, true);
	} else {
		success = false;
	}

	char *logtxt = new char[512];

	if (success) {

		int dims[6] = {srf->NrOfVertices, srf->NrOfTriangles, 1, 1, 1, 1};
		gifti_image *giftiimg = gifti_create_image(2, NIFTI_INTENT_POINTSET, NIFTI_TYPE_FLOAT32, 6, dims, 0);
		giftiimg->numDA = 2;
		giftiimg->compressed = 0;

// ~~~~~~~ add generic meta data
		gifti_add_to_meta(&giftiimg->meta,"GIFTI plugin version", pluginversion, 1);

// ~~~~~~~ tag LabelTable (not used)
		/*
		typedef struct {
			int     length;
			int   * index;
			char ** label;
		} giiLabelTable;

		ideeen: vullen met kleuren of brain structure labels
		 possibility 1: colors in BV's 'int *VertexColor' are indices 0 or 1, referring to Base1/2ColorR/G/B
		 
		 possibility 2: colors in BV's 'int *VertexColor' are defined in the VertexColor itself, defined by: 
						startval + red * pow(16,4) + green * pow(16,2) + blue;

		 
		*/
		if (doExportColors) {
			success = convertMeshColorsToLabelTable(srf);
		}
		

// ~~ fill DataArray[0] with vertex data

		// attributes of DataArray tag
		giftiimg->darray[0]->intent			= NIFTI_INTENT_POINTSET;
		giftiimg->darray[0]->datatype		= NIFTI_TYPE_FLOAT32;
		giftiimg->darray[0]->ind_ord		= GIFTI_IND_ORD_ROW_MAJOR;    // lowest Dim to highest, or reverse
		giftiimg->darray[0]->num_dim		= 2;    // level of DimX applied
		giftiimg->darray[0]->dims[0]		= srf->NrOfVertices;
		giftiimg->darray[0]->dims[1]		= 3;
		giftiimg->darray[0]->encoding		= GIFTI_ENCODING_ASCII;   // format of Data on disk
		giftiimg->darray[0]->endian			= GIFTI_ENDIAN_LITTLE;     // endian, if binary Encoding
		giftiimg->darray[0]->ext_fname		= "";  // external filename, in cur directory
		giftiimg->darray[0]->ext_offset		= 0; // offset of data within external file
		giftiimg->darray[0]->nbyper = 4;
		giftiimg->darray[0]->nvals = srf->NrOfVertices * 3;


		// tag MetaData
		char *name1 = "AnatomicalStructurePrimary";
		char *value1 = getAnatomicalStructurePrimary(srf->origName);
		gifti_add_to_meta(&giftiimg->darray[0]->meta, name1, value1,1);

 		char *name2 = "AnatomicalStructureSecondary";
 		char *value2 = getAnatomicalStructureSecondary(srf->origName);
		gifti_add_to_meta(&giftiimg->darray[0]->meta, name2, value2,1);

 		char *name3 = "GeometricType";
 		char *value3 = getGeometricType(srf->origName);
		gifti_add_to_meta(&giftiimg->darray[0]->meta, name3, value3,1);


		// tag CoordinateSystemTransformMatrix
		gifti_add_empty_CS(giftiimg->darray[0]);
		giftiimg->darray[0]->coordsys[0]->dataspace = nifti_xform_string(NIFTI_XFORM_TALAIRACH);
		giftiimg->darray[0]->coordsys[0]->xformspace = nifti_xform_string(NIFTI_XFORM_TALAIRACH);
		for (int row = 0; row<4; row++)
			for (int col = 0; col<4; col++)
				giftiimg->darray[0]->coordsys[0]->xform[row][col] = (row == col ? 1.0 : 0.0);

		// tag Data
		float *vertexblock = new float[srf->NrOfVertices*3];
		for (int vx = 0; vx < srf->NrOfVertices; vx++) {
			vertexblock[3*vx] =  srf->VertexX[vx];
			vertexblock[3*vx+1] = srf->VertexY[vx];
			vertexblock[3*vx+2] = srf->VertexZ[vx];
		}
		giftiimg->darray[0]->data = vertexblock;



// ~~ fill DataArray[1] with Triangle data

		// attributes of DataArray tag
		giftiimg->darray[1]->intent			= NIFTI_INTENT_TRIANGLE;
		giftiimg->darray[1]->datatype		= NIFTI_TYPE_INT32;
		giftiimg->darray[1]->ind_ord		= GIFTI_IND_ORD_ROW_MAJOR;    // lowest Dim to highest, or reverse
		giftiimg->darray[1]->num_dim		= 2;    // level of DimX applied
		giftiimg->darray[1]->dims[0]		= srf->NrOfTriangles;
		giftiimg->darray[1]->dims[1]		= 3;
		giftiimg->darray[1]->encoding		= GIFTI_ENCODING_ASCII;   // format of Data on disk
		giftiimg->darray[1]->endian			= GIFTI_ENDIAN_LITTLE;     // endian, if binary Encoding
		giftiimg->darray[1]->ext_fname		= "";  // external filename, in cur directory
		giftiimg->darray[1]->ext_offset		= 0; // offset of data within external file
		giftiimg->darray[1]->nbyper = 4;
		giftiimg->darray[1]->nvals = srf->NrOfTriangles * 3;
		//vanwege: da->nvals < 0 || da->nbyper

		// tag MetaData
		char *name4 = "TopologicalType";
		char *value4 = getTopologicalType(srf->origName);
		gifti_add_to_meta(&giftiimg->darray[1]->meta, name4, value4, 1);

		// tag Data
		int *triangleblock = new int[srf->NrOfTriangles*3];

		for (int tx = 0; tx<srf->NrOfTriangles*3; tx++) {
			triangleblock[tx] = srf->triangleSequence[tx];
		}
		giftiimg->darray[1]->data = triangleblock;

		// DataArray to gifti_image (set pointers)
// ~~~~~~~ write GIFTI to disk
		char *giimgname = exchangeExtension(srf->origName, ".surf.gii");
		int success =  gifti_write_image(giftiimg, giimgname, 1);
		if (success == 0) {
			sprintf(logtxt, "Exported to GIfTI format, name:\n %s", giimgname);
			qxLogText(logtxt);
		} else {
			qxLogText("Export to GIfTI format did not succeed.");
			success = false;
		}
	}
	return success;
}

bool export_mtc(char *filename, bool exportfromdisk, bool exportToBinary){

	bool success = true;
	mtcStruct *mtc = getmtc(filename, exportfromdisk);
	if ((int)mtc->fileVersion != -1) {
		headerdump_mtc(mtc, false, true);
	} else {
		success = false;
	}

	if (success) {

		int dims[6] = {mtc->nrOfVertices, 1, 1, 1, 1, 1};
		gifti_image *giftiimg = gifti_create_image(mtc->nrOfVolumes, NIFTI_INTENT_TIME_SERIES, NIFTI_TYPE_FLOAT32, 6, dims, 0);
		giftiimg->numDA = mtc->nrOfVolumes;
		giftiimg->compressed = 0;

		// ~~~~~~~ add generic meta data
		gifti_add_to_meta(&giftiimg->meta,"GIFTI plugin version", pluginversion, 1);		
		char *tr = new char[10];
		sprintf(tr, "%f", mtc->TR);
		gifti_add_to_meta(&giftiimg->meta,"TimeStep",tr,1);

		// ~~~~~~~ tag LabelTable (not used)
		/*
		typedef struct {
		int     length;
		int   * index;
		char ** label;
		} giiLabelTable;

		ideeen: vullen met kleuren of brain structure labels
		*/

		// ~~ fill DataArray[0] with vertex data

		// ~~~~~ data, loop over volumes ~~~~~

		int volnr;
		char *txt = new char[256];
		/*
		 timepoint=zeros(nrvertices*3,1)
		 counter = 1;
		 for vertexnr =1:5
		 for volnr=1:3
		 pos = (volnr-1)*nrvertices
		 if pos < 0, pos=0; end
		 timepoint(counter) = data(pos + vertexnr)
		 %    timepoint[vertexnr] = mtc->data[(volnr*nrvertices + vertexnr)];
		 counter = counter + 1
		 end
		 end 
		 */
		
		
		for (volnr=0; volnr < mtc->nrOfVolumes; volnr++) {

			sprintf(txt, "Adding mesh time course of volume: %d", (volnr + 1));
			qxLogText(txt);

			// attributes of DataArray tag
			giftiimg->darray[volnr]->intent			= NIFTI_INTENT_TIME_SERIES;
			giftiimg->darray[volnr]->datatype		= NIFTI_TYPE_FLOAT32;
			giftiimg->darray[volnr]->ind_ord		= GIFTI_IND_ORD_ROW_MAJOR;    // lowest Dim to highest, or reverse
			giftiimg->darray[volnr]->num_dim		= 1;    // level of DimX applied // changed from 2 to 1 at 31-01-11
			giftiimg->darray[volnr]->dims[0]		= mtc->nrOfVertices;
		//	giftiimg->darray[volnr]->dims[1]		= 1; // canceled 31-01-11
			giftiimg->darray[volnr]->encoding		= GIFTI_ENCODING_ASCII;   // format of Data on disk
			giftiimg->darray[volnr]->endian			= GIFTI_ENDIAN_LITTLE;     // endian, if binary Encoding
			giftiimg->darray[volnr]->ext_fname		= "";  // external filename, in cur directory
			giftiimg->darray[volnr]->ext_offset		= (long long)0; // offset of data within external file
			giftiimg->darray[volnr]->nbyper			= 4;
			giftiimg->darray[volnr]->nvals			= mtc->nrOfVertices;

			int nrvertices = mtc->nrOfVertices;
			float *timepoint = new float[nrvertices];
			int vertexnr;
			for (vertexnr =0; vertexnr < nrvertices; vertexnr++) {
			//	timepoint[vertexnr] = mtc->data[(vertexnr*mtc->nrOfVolumes + volnr)]; // changed 01-02-11, was not necessary.
				timepoint[vertexnr] = mtc->data[(volnr*nrvertices + vertexnr)]; // 
			}
			// tag Data
			giftiimg->darray[volnr]->data = timepoint;

		} // end of loop over timepoints

		// tag MetaData
		/*	char *name1 = "AnatomicalStructurePrimary";
		char *value1 = getAnatomicalStructurePrimary(srf->origName);
		gifti_add_to_meta(&giftiimg->darray[0]->meta, name1, value1,1);

		char *name2 = "AnatomicalStructureSecondary";
		char *value2 = getAnatomicalStructureSecondary(srf->origName);
		gifti_add_to_meta(&giftiimg->darray[0]->meta, name2, value2,1);

		char *name3 = "GeometricType";
		char *value3 = getGeometricType(srf->origName);
		gifti_add_to_meta(&giftiimg->darray[0]->meta, name3, value3,1);
	*/

		// DataArray to gifti_image (set pointers)
		// ~~~~~~~ write GIFTI to disk
		char *giimgname = exchangeExtension(mtc->origName, ".time.gii");
		int success =  gifti_write_image(giftiimg, giimgname, 1);
		if (success == 0) {
			char *logtxt = new char[512];
			sprintf(logtxt, "Exported to GIfTI format, name:\n %s", giimgname);
			qxLogText(logtxt);
		} else {
			qxLogText("Export to GIfTI format did not succeed.");
			success = false;
		}
	}

	return success;
}

bool export_smp(char *filename, bool exportfromdisk, bool exportToBinary){

	bool success = true;
	smpStruct *smp = getsmp(filename, exportfromdisk);
	if (smp->versionNr != -1.0) {
		headerdump_smp(smp, false, true);
	} else {
		success = false;
	}
	

	char *logtxt = new char[512];
	if (success) {

		// retinotopy (since v1.5)
	//	for (volnr=0; volnr < smp->nrOfMaps; volnr++) {
			if (success && (smp->srfStatMaps[0].mapType == BVQX_MAPTYPE_CROSSCORRELATION)) {
				
				char instrRangeMin[] = "Cross-correlation data detected. Please select the map type:\n\n1 = Cross-correlation\n2 = Polar angle\n3 = Eccentricity\n";
				int minRange = qxGetIntegerInput(instrRangeMin, 1, 1, 3);
				if (minRange == 2) {
					smp->srfStatMaps[0].mapType = BVQX_MAPTYPE_RETINOTOPY_POLAR;
				} else if (minRange == 3) {
					smp->srfStatMaps[0].mapType = BVQX_MAPTYPE_RETINOTOPY_ECCENTRICITY;
				} else if (minRange < 1) {
					success = false; // someone clicked Cancel button
				}
			} 
	//	}
		
		gifti_image *giftiimg;
		int dims[6] = {smp->nrOfVertices, 1, 1, 1, 1, 1};
		if ((smp->srfStatMaps[0].mapType == BVQX_MAPTYPE_RETINOTOPY_POLAR) || (smp->srfStatMaps[0].mapType == BVQX_MAPTYPE_RETINOTOPY_ECCENTRICITY)) {
			giftiimg = gifti_create_image(smp->nrOfMaps, NIFTI_INTENT_TTEST, NIFTI_TYPE_COMPLEX64, 6, dims, 0);
		} else {
			giftiimg = gifti_create_image(smp->nrOfMaps, NIFTI_INTENT_TTEST, NIFTI_TYPE_FLOAT32, 6, dims, 0);
		}

		giftiimg->numDA = smp->nrOfMaps;
		giftiimg->compressed = 0;

		// ~~~~~~~ add generic meta data
		gifti_add_to_meta(&giftiimg->meta,"GIFTI plugin version", pluginversion, 1);

		// ~~~~~ data, loop over volumes ~~~~~
		int volnr;
		char *txt = new char[256];
		for (volnr=0; volnr < smp->nrOfMaps; volnr++) {

			
			sprintf(txt, "Adding map number: %d", (volnr + 1));
			qxLogText(txt);

			// METADATA of DataArray: attributes of DataArray tag
			giftiimg->darray[volnr]->intent			= getGIfTIIntent(smp->srfStatMaps[volnr].mapType);			
			
	/*		if ((smp->mapType == BVQX_MAPTYPE_RETINOTOPY_POLAR) || (smp->mapType == BVQX_MAPTYPE_RETINOTOPY_ECCENTRICITY)) {
				giftiimg->darray[volnr]->datatype		= NIFTI_TYPE_COMPLEX64;
			} else {
				giftiimg->darray[volnr]->datatype		= NIFTI_TYPE_FLOAT32;
			}
	*/		giftiimg->darray[volnr]->ind_ord		= GIFTI_IND_ORD_ROW_MAJOR;    // lowest Dim to highest, or reverse
			giftiimg->darray[volnr]->num_dim		= 1;    // level of DimX applied
			giftiimg->darray[volnr]->dims[0]		= smp->nrOfVertices;
	//		giftiimg->darray[volnr]->dims[1]		= 1;
			giftiimg->darray[volnr]->encoding		= GIFTI_ENCODING_ASCII;   // format of Data on disk
			giftiimg->darray[volnr]->endian			= GIFTI_ENDIAN_LITTLE;     // endian, if binary Encoding
			giftiimg->darray[volnr]->ext_fname		= "";  // external filename, in cur directory
			giftiimg->darray[volnr]->ext_offset		= 0; // offset of data within external file
			if ((smp->srfStatMaps[volnr].mapType == BVQX_MAPTYPE_RETINOTOPY_POLAR) || (smp->srfStatMaps[volnr].mapType == BVQX_MAPTYPE_RETINOTOPY_ECCENTRICITY)) {
				giftiimg->darray[volnr]->nbyper			= 8;
			} else {
				giftiimg->darray[volnr]->nbyper			= 4;
			}

			giftiimg->darray[volnr]->nvals			= smp->nrOfVertices;

			// DATA: data of tag DataArray
			if ((smp->srfStatMaps[volnr].mapType == BVQX_MAPTYPE_RETINOTOPY_POLAR) || (smp->srfStatMaps[volnr].mapType == BVQX_MAPTYPE_RETINOTOPY_ECCENTRICITY)) {
								
				std::complex<float> *cdata = crossCorrelationToComplex(smp); 
				giftiimg->darray[volnr]->data = cdata;
				
			} else {
				giftiimg->darray[volnr]->data =  smp->srfStatMaps[volnr].mapData;
			}
			
			// add mapname to data array meta data - added 18-10-11
			char *mapname = new char[256];
			sprintf(mapname, "%s", smp->srfStatMaps[volnr].surfaceMapName);
			gifti_set_DA_meta(giftiimg, "Name", mapname, &volnr, 1, 0);

			// add intent code (statistical distribution) - added 01-02-11
			char *valuetag = new char[25];
			if (getnrofdistributionparams(smp->srfStatMaps[volnr].mapType) > 0){
				sprintf(valuetag, "%i", smp->srfStatMaps[volnr].df1); // is number of lags
				gifti_set_DA_meta(giftiimg, "intent_p1", valuetag, &volnr, 1, 0);
			}
			if (getnrofdistributionparams(smp->srfStatMaps[volnr].mapType) > 1){
				sprintf(valuetag, "%i", smp->srfStatMaps[volnr].df2); 
				gifti_set_DA_meta(giftiimg, "intent_p2", valuetag, &volnr, 1, 0); // '&volnr' = "array", '1' = length of array
			}
			
			if ((smp->srfStatMaps[volnr].mapType == BVQX_MAPTYPE_RETINOTOPY_POLAR) || 
				(smp->srfStatMaps[volnr].mapType == BVQX_MAPTYPE_RETINOTOPY_ECCENTRICITY) ||
				(smp->srfStatMaps[volnr].mapType == BVQX_MAPTYPE_CROSSCORRELATION)) {

				sprintf(valuetag, "%i", smp->srfStatMaps[0].minLag); 
				gifti_set_DA_meta(giftiimg, "minLag", valuetag, &volnr, 1, 0);
			}

		} // end of loop over timepoints

		// ~~~~~~~ write GIFTI to disk
		char *giimgname = exchangeExtension(smp->origName, ".func.gii");

		qxLogText(giimgname);

		int success =  gifti_write_image(giftiimg, giimgname, 1);
		if (success == 0) {
			sprintf(logtxt, "Exported to GIfTI format, name:\n %s", giimgname);
			qxLogText(logtxt);
		} else {
			qxLogText("Export to GIfTI format did not succeed.");
			success = false;
		}
	}

	return success;
}


bool export_poi(char *filename, bool exportfromdisk, bool exportToBinary) { 

	bool success = true;
	poiStruct *poi = getpoi(filename, exportfromdisk);
	if (poi->FileVersion != -1.0) {
		headerdump_poi(poi, false, true);
	} else {
		success = false;
	}
	
	char *logtxt = new char[512];
	
	if (success) {
		
		// maak GIFTI image aan
		int dims[6] = {poi->NrOfMeshVertices, 1, 1, 1, 1, 1};
		gifti_image *giftiimg = gifti_create_image(2, NIFTI_INTENT_LABEL, NIFTI_TYPE_INT32, 6, dims, 0);
		giftiimg->numDA = 2;
		giftiimg->compressed = 0;
		
		// ~~~ add metadata
		gifti_add_to_meta(&giftiimg->meta,"GIFTI plugin version", pluginversion, 1);
		gifti_add_to_meta(&giftiimg->meta,"Corresponding (underlying) BrainVoyager surface", poi->FromMeshFile,1);
		char *nrvertices = new char[10];
		sprintf(nrvertices, "%i", poi->NrOfMeshVertices);
		gifti_add_to_meta(&giftiimg->meta,"Number of mesh vertices", nrvertices,1);
		
		// add all vertices
		int vertexcounter = 0;
		for (int i = 0; i < poi->NrOfPOIs; i++) {
			vertexcounter += poi->poi[i].NrOfVertices;
		}
		
		// attributes of DataArray tag 'Node index'
		giftiimg->darray[0]->intent			= NIFTI_INTENT_NODE_INDEX;
		giftiimg->darray[0]->datatype		= NIFTI_TYPE_INT32;
		giftiimg->darray[0]->ind_ord		= GIFTI_IND_ORD_ROW_MAJOR;    // lowest Dim to highest, or reverse
		giftiimg->darray[0]->num_dim		= 1;    // level of DimX applied
		giftiimg->darray[0]->dims[0]		= vertexcounter;
		giftiimg->darray[0]->encoding		= GIFTI_ENCODING_ASCII;   // format of Data on disk
		giftiimg->darray[0]->endian			= GIFTI_ENDIAN_LITTLE;     // endian, if binary Encoding
		giftiimg->darray[0]->ext_fname		= "";  // external filename, in cur directory
		giftiimg->darray[0]->ext_offset		= 0; // offset of data within external file
		giftiimg->darray[0]->nbyper			= 4;
		giftiimg->darray[0]->nvals			= vertexcounter;
		
		
		// maak node index
		int index=0;
		int *nodeindexblock = new int[vertexcounter];
		for (int i = 0; i < poi->NrOfPOIs; i++) {
			for (int j = 0; j < poi->poi[i].NrOfVertices; j++) {
				nodeindexblock[index] = poi->poi[i].POIdata[j];
				index++;
			}
		}
		giftiimg->darray[0]->data = nodeindexblock;
		
		
		// maak table met labels van de POIs: de namen van de hersengebieden
		int color, startval	= 0;//1056964608;
		float red, green, blue, alpha;
		// ~~~ convert RGBA colors in huge number to 0.0-1.0.
		giftiimg->labeltable.length = poi->NrOfPOIs;
		float *rgba = new float[poi->NrOfPOIs*4];
		int *key = new int [poi->NrOfPOIs];
		char **label = new char*[poi->NrOfPOIs];
		for(int i = 0; i<poi->NrOfPOIs; i++) label[i] = new char[256];// in docu: lengte is 256
		
		//allocate space for sparse indices with maximum possible size 
		for (int i = 0; i<poi->NrOfPOIs; i++){
			color = poi->poi[i].ColorOfPOI;
			color -= startval;
			red = (int)(color/pow(16.0f,4));
			green = (int) ( (color-red*pow(16.0f,4)) /pow(16.0f,2));
			blue = (int)(color-red*pow(16.0f,4)-green*pow(16.0f,2));
			red /= 255;
			green /= 255;
			blue /= 255;
			key[i] = i;
			sprintf(label[i], poi->poi[i].POIName);
			rgba[i*4] = red;
			rgba[1+i*4] = green;
			rgba[2+i*4] = blue;
			rgba[3+i*4] = 1.0f; //alpha value
			
		}
		giftiimg->labeltable.rgba = rgba;
		giftiimg->labeltable.key = key;
		giftiimg->labeltable.label = label;
		
		// attributes of DataArray tag 'Label'
		giftiimg->darray[1]->intent			= NIFTI_INTENT_LABEL;
		giftiimg->darray[1]->datatype		= NIFTI_TYPE_INT32;
		giftiimg->darray[1]->ind_ord		= GIFTI_IND_ORD_ROW_MAJOR;    // lowest Dim to highest, or reverse
		giftiimg->darray[1]->num_dim		= 1;    // level of DimX applied
		giftiimg->darray[1]->dims[0]		= vertexcounter;
		giftiimg->darray[1]->encoding		= GIFTI_ENCODING_ASCII;   // format of Data on disk
		giftiimg->darray[1]->endian			= GIFTI_ENDIAN_LITTLE;     // endian, if binary Encoding
		giftiimg->darray[1]->ext_fname		= "";  // external filename, in cur directory
		giftiimg->darray[1]->ext_offset		= 0; // offset of data within external file
		giftiimg->darray[1]->nbyper			= 4;
		giftiimg->darray[1]->nvals			= vertexcounter;
		
		int *colorkeyblock = new int[vertexcounter];
		index=0;
		for (int i = 0; i < poi->NrOfPOIs; i++) {
			for (int j = 0; j < poi->poi[i].NrOfVertices; j++) {
				colorkeyblock[index] = i;
				index++;
			}
		}
		giftiimg->darray[1]->data = colorkeyblock;
		
		// ~~~~~~~ write GIFTI to disk
		char *giimgname = exchangeExtension(poi->POIsFileName, ".label.gii");
		int success =  gifti_write_image(giftiimg, giimgname, 1);
		if (success == 0) {
			sprintf(logtxt, "Exported to GIfTI format, name:\n %s", giimgname);
			qxLogText(logtxt);
		} else {
			qxLogText("Export to GIfTI format did not succeed.");
			success = false;
		}
	}
	return success;
}
