

/*!
@header gifti1file.h
@discussion header for surface model data (*.srf files) 
*/

#ifndef GIFTI1FILE_H
#define GIFTI1FILE_H

#include "BVQXPluginInterfaceExtern.h"
#include "gifti_io.h"


/*!
@class gifti1file
@superclass ... (not yet implemented)
@abstract contains functions to deal with GIfTI surface files (*.gii)
@version 0.1
*/
class gifti1file { //: public ... {

	float			*data;

	/*!
	@var vmrmainheader
	@abstract pointer to VMR_Header of BVQXPluginAccessInterface
	*/
	//struct 

	public:

	gifti_image *gifti1mainheader;

	
	


	/*! 
	@function gifti1file
	@abstract constructor
	*/
	gifti1file();
	/*! 
	@function ~gifti1file
	@abstract destructor
	*/
	~gifti1file();

	/*! 
	@function initheader
	@abstract initialize header
	@discussion not used
	*/
	void	initheader();
	/*! 
	@function readheader
	@abstract read header from disk
	@discussion 
	*/
	bool	readheader(char *filename);
	/*! 
	@function writeheader
	@abstract write dimensions to disk in binary format
	@discussion nonsense function, is relic from past 
	*/
	bool	writeheader();
	/*! 
	@function getcurrentheader
	@abstract get headers of VMR that is currently opened in BrainVoyager QX
	@discussion uses giftilib access functions
	*/
	bool	getcurrentheader();
	/*! 
	@function getcurrentdata
	@abstract get data of SRF that is currently opened in BrainVoyager QX
	@discussion uses BVQX plugin access functions, e.g. qxGetCurrentVMR(), converts the obtained unsigned char data to float
	and saves them in the neuroimagefile field 'fdata'
	*/
	bool	getcurrentdata();
	/*!
	@function header2ascii
	@abstract print SRF header values to character array
	@result SRF header values
	*/
	char	*header2ascii();
	/*!
	@function header2ascii
	@abstract print header values to character array
	@param filename (cstr)
	@result GIfTI header values
	*/
	char	*header2ascii(char *filename);
	/*! 
	@function loaddata
	@abstract get data of 
	@discussion forwards to function getcurrentdata()
	*/
	bool	loaddata();
	/*!
	@function copyheader
	@abstract copies local fields to generic neuroimagefile fields or the reverse
	@param fromheader2header indicates whether to copy from the file specific header fields to neuroimage header fields or the reverse
	@version 0.1
	*/
	bool	copyheader(short int fromheadertoheader);
	/*!
	@function write
	@abstract write GIfTI-1 file to disk
	@discussion 
	*/
	bool	write();

};

#endif // GIFTI1FILE_H