/*
#if defined(OS_WIN32)
#include <windows.h>
#include <malloc.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <fstream>

#include <string.h>
#include <math.h>
#include <ctype.h>
#include <exception>

/// bvqx specific files ///
#include "global.h"
*/
#include "BVQXPluginInterfaceExtern.h"
/*#include "giftiplugin.h"
*/

// plugin specific files 
#include "gifti_io.h"
#include "bvqx_io.h"
#include "gifti1file.h"
#include "SRF_Header.h"
#include "MTC_Header.h"
#include "SMP_Header.h"
#include "convertbvqxstructs.h"
#include "info.h"


/*
void error_message(int errortype, int filetype) {

	char *info = new char[512];
	switch(errortype) {
	case ERROR_TYPE_UNDEFINED:
		sprintf(info, "An error occurred. Procedure finished.");
		break;
	case ERROR_TYPE_FILE_SEL:
		sprintf(info, "No %s file was selected. Procedure finished.", get_bvqx_datatype_str(filetype));
		break;
	default:
		sprintf(info, "An error occurred. Procedure finished.");
	}
	qxMessageBox(info);
	qxLogText(info);

}
*/
bool getsrfinfo(bool exportfromdisk, bool write2disk, bool write2log) {
	
	char *filename = new char [256];
	if (exportfromdisk) {
		int usrresp = qxGetOpenFileName("Please select the BrainVoyager surface file", "", "*.srf", filename);
		if (usrresp < 1) {
			qxMessageBox("No surface file was selected. Procedure finished.");
			return false;
		}
	} else sprintf(filename, "none");
	srfStruct *srf = new srfStruct;
	srf = getsrf(filename, exportfromdisk);
	if ((int)srf->fileVersion != -1) {
		headerdump_srf(srf, write2disk, write2log);
	}
	delete srf; 
	return true;
}

bool getmtcinfo(bool exportfromdisk, bool write2disk, bool write2log) {

	char *filename = new char [256];
	if (exportfromdisk) {
		int usrresp = qxGetOpenFileName("Please select the BrainVoyager Mesh Time Course file", "", "*.mtc", filename);
		if (usrresp < 1) {
			qxMessageBox("No Mesh Time Course file was selected. Procedure finished.");
			return false;
		}
	} else sprintf(filename, "none");
	mtcStruct *mtc = new mtcStruct;
	mtc = getmtc(filename, exportfromdisk);
	if (mtc->fileVersion != -1) {
		headerdump_mtc(mtc, write2disk, write2log);
	}
	delete mtc; 
	return true;
}

bool getsmpinfo(bool exportfromdisk, bool write2disk, bool write2log) {

	char *filename = new char [256];
	if (exportfromdisk) {
		int usrresp = qxGetOpenFileName("Please select the BrainVoyager Surface Map", "", "*.smp", filename);
		if (usrresp < 1) {
			qxMessageBox("No Mesh Time Course file was selected. Procedure finished.");
			return false;
		}
	} else sprintf(filename, "none");

	smpStruct *smp = new smpStruct;
	smp = getsmp(filename, exportfromdisk);
	if (smp->versionNr != -1) {
		headerdump_smp(smp, write2disk, write2log);
	}
	delete smp; 
	return true;
}

bool getpoiinfo(bool exportfromdisk, bool write2disk, bool write2log) {
	
	char *filename = new char[256];
	if (exportfromdisk) {
		int usrresp = qxGetOpenFileName("Please select the BrainVoyager POI file", "", "*.poi", filename);
		if (usrresp < 1) {
			qxMessageBox("No POI file was selected. Procedure finished.");
			return false;
		}
	} else sprintf(filename, "none");
	
	poiStruct *poi = new poiStruct;
	poi = getpoi(filename, exportfromdisk);
	if (poi->FileVersion != -1) {
		headerdump_poi(poi, write2disk, write2log);
	}
	delete poi; 
	return true;
}

bool getgiftiinfo(bool write2disk, bool write2log) {

	char *filename = new char [256];
	int usrresp = qxGetOpenFileName("Please select the GIFTI file", "", "*.gii", filename);
	if (usrresp < 1) {
		qxMessageBox("No GIFTI file was selected. Procedure finished.");
		return false;
	}
	gifti_image *giftiimg = gifti_read_image(filename, 0);
	if (giftiimg != NULL) {
		headerdump_gifti(giftiimg, write2disk, write2log, filename);
	}
	delete giftiimg; 
	return false;
}

void headerdump_gifti(gifti_image *giftiimg, bool write2disk, bool write2log, char *filename) {

//	gifti_image *giftiimg = gifti_read_image(filename, 0); // uit per 25-11
//	char **proptxt = giftiimg->meta.name;
//	qxLogText(*proptxt);

	char *buf; int bufLen; 
	buf = (char *)calloc(1, giftiimg->numDA*5000); bufLen = 0;  
	
	try {

		giiDataArray *arr;
		sprintf(buf, "\nGIfTI filename: \n%s\n", filename); 	

		// Print MetaData 
		sprintf(buf+strlen(buf), "Number of MetaData tags: %d\n", giftiimg->meta.length);
		for (int mmdtag = 0; mmdtag < giftiimg->meta.length; mmdtag++){
			sprintf(buf+strlen(buf), "Number: %d\n", mmdtag);
			sprintf(buf+strlen(buf), "Name: %s\n", giftiimg->meta.name[mmdtag]);
			sprintf(buf+strlen(buf), "Value: %s\n", giftiimg->meta.value[mmdtag]);
		}


		for(int DAnr=0; DAnr<giftiimg->numDA; DAnr++){

			arr = giftiimg->darray[DAnr];
			// Print attributes of DataArray
			sprintf(buf+strlen(buf), "\nDataArray %i\nGIfTI intent: \n%s\n", (DAnr+1), gifti_intent_to_string(arr->intent));
			sprintf(buf+strlen(buf), "GIfTI data type: \n%s\n", gifti_datatype2str(arr->datatype));
			sprintf(buf+strlen(buf), "ArrayIndexingOrder: %d\n", arr->ind_ord);
			sprintf(buf+strlen(buf), "Dimensionality: %d\n", arr->num_dim);
			sprintf(buf+strlen(buf), "Dim0: %d\n", arr->dims[0]);
			sprintf(buf+strlen(buf), "Dim1: %d\n", arr->dims[1]);
			if (arr->num_dim > 2) sprintf(buf+strlen(buf), "Dim2: %d\n", arr->dims[2]);
			sprintf(buf+strlen(buf), "Encoding: %s\n", gifti_list_index2string(gifti_encoding_list,arr->encoding));		
			sprintf(buf+strlen(buf), "Endian: %s\n", gifti_list_index2string(gifti_endian_list, arr->endian));
			sprintf(buf+strlen(buf), "External file name: %s\n", arr->ext_fname);
			sprintf(buf+strlen(buf), "External file offset: %ld\n", arr->ext_offset);

			// Print MetaData of DataArray
			sprintf(buf+strlen(buf), "Number of MetaData tags of DataArray: %d\n", arr->meta.length);
			for (int mdtag = 0; mdtag < arr->meta.length; mdtag++){
				sprintf(buf+strlen(buf), "Number: %d\n", mdtag);
				sprintf(buf+strlen(buf), "Name: %s\n", arr->meta.name[mdtag]);
				sprintf(buf+strlen(buf), "Value: %s\n", arr->meta.value[mdtag]);
			}
			
			// Print CoordinateSystemTransformMatrix information
			if (arr->intent == NIFTI_INTENT_POINTSET) {
				giiCoordSystem *coordsys = arr->coordsys[0];
				sprintf(buf+strlen(buf), "Data space: %s\n", coordsys->dataspace); //arr->coordsys->dataspace[0]);
				sprintf(buf+strlen(buf), "XForm space: %s\n", coordsys->xformspace);
				sprintf(buf+strlen(buf), "Matrix data:\n");
				int i,j;
				for (i=0; i< 4; i++) {
					for (j=0; j < 4; j++) {
						sprintf(buf+strlen(buf), "\t%f", coordsys->xform[i][j]);
					}
					sprintf(buf+strlen(buf), "\n");
				}
			}
		}
		bufLen = strlen(buf);
		buf  = (char *)realloc((void *)buf, bufLen+1);
		if (write2disk){
			FILE *fileptr;
			char *name = exchangeExtension(filename,"_info.txt");
			fileptr = fopen(name, "w");
			fputs(buf,fileptr);
			fclose(fileptr);
			qxLogText("Wrote the header info to disk.");
		}
		if (write2log) qxLogText(buf);

	} catch (...) {
		qxLogText("Error while printing GIfTI header data to ascii");
	}				
	free(buf);
}

// nieuw
//	int gifti_disp_gifti_image(const char * mesg, const gifti_image * p, int subs)




void headerdump_srf(srfStruct *srf, bool write2disk, bool write2log) {

	char *buf; int bufLen; 
	buf = (char *)calloc(1,65534); bufLen = 0;  

	try {

		//sprintf(buf, "\nSRF filename: \n%s\n", filename);
		sprintf(buf, "\nSRF filename: \n%s\n", srf->origName);
		sprintf(buf+strlen(buf), "File type: surface (*.srf)\n");
		sprintf(buf+strlen(buf), "File version: %1.1f\n", srf->fileVersion);
		sprintf(buf+strlen(buf), "Number of vertices: %d\n", srf->NrOfVertices);
		sprintf(buf+strlen(buf), "Number of triangles: %d\n", srf->NrOfTriangles);
		sprintf(buf+strlen(buf), "Mesh center: X: %f, Y: %f, Z: %f\n", srf->MeshCenterX, srf->MeshCenterY, srf->MeshCenterZ);
		sprintf(buf+strlen(buf), "Mesh curvature colors: \n");
		sprintf(buf+strlen(buf), "Concave red: %f, green: %f, blue: %f\n", srf->ConcCurvRed, srf->ConcCurvGreen, srf->ConcCurvBlue);
		sprintf(buf+strlen(buf), "Concave alpha: %f\n", srf->ConcCurvAlpha);
		sprintf(buf+strlen(buf), "Convex red: %f, green: %f, blue: %f\n", srf->ConvCurvRed, srf->ConvCurvGreen, srf->ConvCurvBlue);
		sprintf(buf+strlen(buf), "Convex alpha: %f\n", srf->ConvCurvAlpha);

		bufLen = strlen(buf);
		buf  = (char *)realloc((void *)buf, bufLen+1);

		if (write2disk){
			FILE *fileptr;
			char *name = exchangeExtension(srf->origName,"_info.txt");
			fileptr = fopen(name, "w");
			fputs(buf,fileptr);
			fclose(fileptr);
			qxLogText("Wrote the header info to disk.");
		}
		if (write2log) qxLogText(buf);

	} catch (...) {
		qxLogText("Error while printing SRF header data to ascii");
	}				

}


void headerdump_mtc(mtcStruct *mtc, bool write2disk, bool write2log) {


	char *buf; int bufLen; 
	buf = (char *)calloc(1,65534); bufLen = 0;

	try {

		sprintf(buf, "\nMTC filename: \n%s\n", mtc->origName); 
		sprintf(buf+strlen(buf), "File type: mesh time course (*.mtc)\n");
		sprintf(buf+strlen(buf), "Fileversion: %d\n", mtc->fileVersion);
		sprintf(buf+strlen(buf), "Number of vertices: %d\n", mtc->nrOfVertices);
		sprintf(buf+strlen(buf), "Number of volumes: %d\n", mtc->nrOfVolumes);
		sprintf(buf+strlen(buf), "VTC filename: %s\n", mtc->vtcFileName);
		sprintf(buf+strlen(buf), "PRT filename: %s\n", mtc->prtFileName);
		sprintf(buf+strlen(buf), "TR in milliseconds: %6.0f\n", mtc->TR);
		sprintf(buf+strlen(buf), "HRF: %d\n", mtc->hemoDynDelay);
		sprintf(buf+strlen(buf), "HRF delta: %f\n", mtc->HRFdelta);
		sprintf(buf+strlen(buf), "HRF tau: %f\n", mtc->HRFtau);
		sprintf(buf+strlen(buf), "Segment size: %d\n", mtc->segmentSize);
		sprintf(buf+strlen(buf), "Segment offset: %d\n", mtc->segmentOffset);

		bufLen = strlen(buf) ;
		buf  = (char *)realloc((void *)buf, bufLen+1);

		if (write2disk){
			FILE *fileptr;
			char *name = exchangeExtension(mtc->origName,"_info.txt");
			fileptr = fopen(name, "wb");
			fputs(buf,fileptr);
			fclose(fileptr);
			qxLogText("Wrote the header info to disk.");
		}
		if (write2log) qxLogText(buf);

	} catch (...) {
		qxLogText("Error while printing MTC header data to ascii");
	}				

	
}

// graag toevoegen nieuwe velden 07-01-11
void headerdump_smp(smpStruct *smp, bool write2disk, bool write2log) {

		char *buf = new char[65534]; int bufLen = 0; 
	//	buf = (char *)calloc(1,65534); bufLen = 0;

		try {

			sprintf(buf, "\nSMP filename: \n%s\n", smp->origName); 
			sprintf(buf+strlen(buf), "File type: surface map (*.smp)\n");
			sprintf(buf+strlen(buf), "File version: %d\n", smp->versionNr);
			sprintf(buf+strlen(buf), "Number of vertices: %d\n", smp->nrOfVertices);
			sprintf(buf+strlen(buf), "Number of maps: %d\n", smp->nrOfMaps);
			sprintf(buf+strlen(buf), "Surface filename: %s\n", smp->srfFileName);

			if (smp->versionNr < 4) { // don't know what will happen after 3
				int mapCntr;
				for (mapCntr = 0; mapCntr < smp->nrOfMaps; mapCntr++) {
					sprintf(buf+strlen(buf), "\nDetails for map nr: %d\n\n", (mapCntr + 1)); 
					sprintf(buf+strlen(buf), "Map type: %s\n",
						(smp->srfStatMaps[mapCntr].mapType == 1) ? "T-test"
						:(smp->srfStatMaps[mapCntr].mapType == 2) ? "Correlation"
						:(smp->srfStatMaps[mapCntr].mapType == 3) ? "Cross-correlation"
						:(smp->srfStatMaps[mapCntr].mapType == 4) ? "F-test"
						:(smp->srfStatMaps[mapCntr].mapType == 11) ? "Percent signal change"
						:"Unknown");
					if (smp->srfStatMaps[mapCntr].mapType == 3) {
						sprintf(buf+strlen(buf), "Number of lags: %i\n", smp->srfStatMaps[mapCntr].nrOfLags);
						sprintf(buf+strlen(buf), "Minimum lag: %i\n", smp->srfStatMaps[mapCntr].minLag);
						sprintf(buf+strlen(buf), "Maximum lag: %i\n", smp->srfStatMaps[mapCntr].maxLag);
						sprintf(buf+strlen(buf), "Cross correlation overlay: %i\n", smp->srfStatMaps[mapCntr].ccOverlay);
						sprintf(buf+strlen(buf), "Contrast X-RC maps: %i\n", smp->srfStatMaps[mapCntr].contrXrcmaps);
					}
					sprintf(buf+strlen(buf), "Clustersize: %d\n", smp->srfStatMaps[mapCntr].clusterSize);
					sprintf(buf+strlen(buf), "Enable clustercheck: %s\n", (smp->srfStatMaps[mapCntr].enableClusterCheck) ? "Yes":"No");
					sprintf(buf+strlen(buf), "Minimum threshold: %f\n", smp->srfStatMaps[mapCntr].critValThresh);
					sprintf(buf+strlen(buf), "Maximum threshold: %f\n", smp->srfStatMaps[mapCntr].maxThresh);	
					sprintf(buf+strlen(buf), "DF 1: %d\n", smp->srfStatMaps[mapCntr].df1);
					sprintf(buf+strlen(buf), "DF 2: %d\n", smp->srfStatMaps[mapCntr].df2);
					sprintf(buf+strlen(buf), "Cortex based Bonferroni: %s\n", (smp->srfStatMaps[mapCntr].bonfCorr)? "Yes":"No");
					sprintf(buf+strlen(buf), "Critical value color: %d %d %d\n", smp->srfStatMaps[mapCntr].RGBLowerThreshPos[0],smp->srfStatMaps[mapCntr].RGBLowerThreshPos[1],smp->srfStatMaps[mapCntr].RGBLowerThreshPos[2]);
					sprintf(buf+strlen(buf), "Maximum value color: %d %d %d\n", smp->srfStatMaps[mapCntr].RGBUpperThreshPos[0],smp->srfStatMaps[mapCntr].RGBUpperThreshPos[1],smp->srfStatMaps[mapCntr].RGBUpperThreshPos[2]);
					sprintf(buf+strlen(buf), "Use VMP color: %s\n", (smp->srfStatMaps[mapCntr].enableSMPcolor)? "Yes":"No");
					sprintf(buf+strlen(buf), "Transparent color factor: %f\n", smp->srfStatMaps[mapCntr].transparentColorFactor);
					sprintf(buf+strlen(buf), "Map name: %s\n", smp->srfStatMaps[mapCntr].surfaceMapName);

				}
			}
			bufLen = strlen(buf);
			buf  = (char *)realloc((void *)buf, bufLen+1);

			if (write2disk){
				FILE *fileptr;
				char *name = exchangeExtension(smp->origName,"_info.txt");
				fileptr = fopen(name, "wb");
				fputs(buf,fileptr);
				fclose(fileptr);
				qxLogText("Wrote the header info to disk.");
			}
			if (write2log) qxLogText(buf);	

		} catch (...) {
			qxLogText("Error while printing SMP header data to ascii");
		}

}

void headerdump_poi(poiStruct *poi, bool write2disk, bool write2log) {
	
	char *buf; int bufLen; 
	buf = (char *)calloc(1,65534); bufLen = 0;
	
	try {
		
		sprintf(buf, "\nPOI filename: \n%s\n", poi->POIsFileName); 
		sprintf(buf+strlen(buf), "File type: Patch of Interest (*.poi)\n");
		sprintf(buf+strlen(buf), "File version: %d\n", poi->FileVersion);
		sprintf(buf+strlen(buf), "From Mesh File: %s\n", poi->FromMeshFile);
		sprintf(buf+strlen(buf), "Number of vertices: %d\n", poi->NrOfMeshVertices);
		sprintf(buf+strlen(buf), "Number of POIs: %d\n", poi->NrOfPOIs);
		sprintf(buf+strlen(buf), "Number of POI MTCs: %d\n", poi->NrOfPOIMTCs);
	
		for (int POInr = 0; POInr < poi->NrOfPOIs; POInr++) {
			sprintf(buf+strlen(buf), "\nDetails for map nr: %d\n\n", (POInr + 1)); 
			sprintf(buf+strlen(buf), "Number of vertices: %d\n", poi->poi[POInr].NrOfVertices);
			sprintf(buf+strlen(buf), "POI name: %s\n", poi->poi[POInr].POIName);
			sprintf(buf+strlen(buf), "Color code: %d\n", poi->poi[POInr].ColorOfPOI);
			sprintf(buf+strlen(buf), "Reference Vertex: %d\n", poi->poi[POInr].ReferenceVertex);
			sprintf(buf+strlen(buf), "Info Text File: %s\n", poi->poi[POInr].InfoFileName);
		}

		bufLen = strlen(buf);
		buf  = (char *)realloc((void *)buf, bufLen+1);
		
		if (write2disk){
			FILE *fileptr;
			char *name = exchangeExtension(poi->POIsFileName,"_info.txt");
			fileptr = fopen(name, "wb");
			fputs(buf,fileptr);
			fclose(fileptr);
			qxLogText("Wrote the header info to disk.");
		}
		if (write2log) qxLogText(buf);	
		
	} catch (...) {
		qxLogText("Error while printing POI header data to ascii");
	}
	
}