#include "SRF_Header.h"
#include "gifti_io.h"
#include "surfacefunctions.h"
#include "bvqx_io.h"
#include "constants.h"
#include <math.h>
#include <map>
#include <set>
#include <complex>



#include "BVQXPluginInterfaceExtern.h" /// MOET WEG!!!!!!!!!!!!!!!!!!!!!!!!!!!!

//using namespace std;

void computeTriangleNormals(srfStruct *srf, float *triangleNormalX, float *triangleNormalY, float *triangleNormalZ)
{
	float p1[3];
	float p2[3];
	float p3[3];
	float a[3];
	float b[3];
	float normfactor;
	int TriangleIndex1, TriangleIndex2, TriangleIndex3;


	for (int triangle_nr = 0; triangle_nr < srf->NrOfTriangles; triangle_nr++)
	{
		TriangleIndex1 = srf->triangleSequence[triangle_nr*3];
		TriangleIndex2 = srf->triangleSequence[triangle_nr*3+1];
		TriangleIndex3 = srf->triangleSequence[triangle_nr*3+2];
		//Find coordinate p1, p2, p3 of triangle consisting of 3 triangle indices
		p1[0] = srf->VertexX[TriangleIndex1];
		p1[1] = srf->VertexY[TriangleIndex1];
		p1[2] = srf->VertexZ[TriangleIndex1];
		p2[0] = srf->VertexX[TriangleIndex2];
		p2[1] = srf->VertexY[TriangleIndex2];
		p2[2] = srf->VertexZ[TriangleIndex2];
		p3[0] = srf->VertexX[TriangleIndex3];
		p3[1] = srf->VertexY[TriangleIndex3];
		p3[2] = srf->VertexZ[TriangleIndex3];
		//cross product (p2-p1)*(p3-p2)
		//a × b = (a2b3 − a3b2, a3b1 − a1b3, a1b2 − a2b1).
		//a = (p2-p1), b = (p3-p2)
        for (int i = 0; i<3; i++)
		{
            a[i] = p2[i]-p1[i];
            b[i] = p3[i]-p2[i];
		}
		triangleNormalX[triangle_nr] = a[1] * b[2] - a[2] * b[1];
		triangleNormalY[triangle_nr] = a[2] * b[0] - a[0] * b[2];
		triangleNormalZ[triangle_nr] = a[0] * b[1] - a[1] * b[0];
		//normalization
		normfactor = sqrtf(powf(triangleNormalX[triangle_nr],2)+
			powf(triangleNormalY[triangle_nr],2)+powf(triangleNormalZ[triangle_nr],2));
		triangleNormalX[triangle_nr] /= normfactor;
		triangleNormalY[triangle_nr] /= normfactor;
		triangleNormalZ[triangle_nr] /= normfactor;
	}
}

bool rewindTriangles(srfStruct *srf) {

    float tmp;

    for (int triangle_nr = 0; triangle_nr < srf->NrOfTriangles; triangle_nr++) {

            tmp = srf->triangleSequence[triangle_nr*3];
            srf->triangleSequence[triangle_nr*3] = srf->triangleSequence[triangle_nr*3+2];
            srf->triangleSequence[triangle_nr*3+2] = tmp;

    }

    return true;
}

bool computeVertexNormals(srfStruct *srf)
{
    char *txt = new char[256];

	float *triangleNormalX = new float [srf->NrOfTriangles];
	float *triangleNormalY = new float [srf->NrOfTriangles];
	float *triangleNormalZ = new float [srf->NrOfTriangles];
		for (int i = 0; i<srf->NrOfTriangles; i++)
	{
		triangleNormalX[i] = 0.0f;
		triangleNormalY[i] = 0.0f;
		triangleNormalZ[i] = 0.0f;
	}
	computeTriangleNormals(srf, triangleNormalX, triangleNormalY, triangleNormalZ);

//	for (int i = 0; i<srf->NrOfTriangles; i++)
//	{
//	    sprintf(txt, "Triangle Normal %i, x: %f, y: %f, z: %f", i, triangleNormalX[i], triangleNormalY[i], triangleNormalZ[i]);
//		qxLogText(txt);
//	}
	float *vertexNormalX = new float [srf->NrOfVertices];
	float *vertexNormalY = new float [srf->NrOfVertices];
	float *vertexNormalZ = new float [srf->NrOfVertices];
	for (int i = 0; i<srf->NrOfVertices; i++)
	{
		vertexNormalX[i] = 0.0f;
		vertexNormalY[i] = 0.0f;
		vertexNormalZ[i] = 0.0f;
	}


	int vertex_nr, triangle_nr;
	for (int t = 0; t<srf->NrOfTriangles*3; t++)
	{
		vertex_nr = srf->triangleSequence[t];
		triangle_nr = t/3;
		vertexNormalX[vertex_nr] += triangleNormalX[triangle_nr];
		vertexNormalY[vertex_nr] += triangleNormalY[triangle_nr];
		vertexNormalZ[vertex_nr] += triangleNormalZ[triangle_nr];
	}

	float normfactor;
	for (vertex_nr = 0; vertex_nr<srf->NrOfVertices; vertex_nr++)
	{
		normfactor = sqrtf(powf(vertexNormalX[vertex_nr],2)+
			powf(vertexNormalY[vertex_nr],2)+powf(vertexNormalZ[vertex_nr],2));
//        sprintf(txt, "Normfactor %i = %f", vertex_nr, normfactor);
//		qxLogText(txt);
		vertexNormalX[vertex_nr] /= normfactor;
		vertexNormalY[vertex_nr] /= normfactor;
		vertexNormalZ[vertex_nr] /= normfactor;
	}
	srf->NormalX = vertexNormalX;
	srf->NormalY = vertexNormalY;
	srf->NormalZ = vertexNormalZ;

	delete [] triangleNormalX;
	delete [] triangleNormalY;
	delete [] triangleNormalZ;

	return true;
}


float meanFloatArray(float *list, int length)
{
	float mean = 0;
	for (int i = 0; i<length; i++)
		mean += list[i];

	mean /= length;

	return mean;
}


bool verticesGii2Srf(srfStruct *srf, gifti_image *gii)
{
	bool success = true;
	// tag Data
	float *vertexblock = (float*)gii->darray[0]->data;
	char *txt = new char[256];
	sprintf(txt, "nr of vertices: %i", srf->NrOfVertices);
	qxLogText(txt);
	try
	{
		float *VertexX = new float [srf->NrOfVertices];
		float *VertexY = new float [srf->NrOfVertices];
		float *VertexZ = new float [srf->NrOfVertices];

		for (int vx = 0; vx < srf->NrOfVertices; vx++) {
			VertexX[vx] = vertexblock[3*vx];
			VertexY[vx] = vertexblock[3*vx+1];
			VertexZ[vx] = vertexblock[3*vx+2];
		}

		srf->VertexX = VertexX;
		srf->VertexY = VertexY;
		srf->VertexZ = VertexZ;

	} catch (...) {
		success = false;
	}
	return success;
}

bool trianglesGii2Srf(srfStruct *srf, gifti_image *gii)
{
	bool success = true;

	int *triangleblock = (int*)gii->darray[1]->data;
	int *triangleSequence = new int[srf->NrOfTriangles*3];

	try
	{
		for (int tx = 0; tx<srf->NrOfTriangles*3; tx++) triangleSequence[tx] = triangleblock[tx];
		srf->triangleSequence = triangleSequence;
	} catch (...) {
		success = false;
	}
	return success;
}

bool countNeighbours(srfStruct *srf, int *nrNeighbours)
{
	for(int i = 0; i<srf->NrOfVertices; i++) nrNeighbours[i] = 0;

	for(int t = 0; t<srf->NrOfTriangles*3; t++)
		nrNeighbours[srf->triangleSequence[t]]++;

	return true;
}

bool collectNeighbours(srfStruct *srf, int **doubleNeighboursList)
{
	int *neighboursFound = new int[srf->NrOfVertices];
	for(int i = 0; i<srf->NrOfVertices; i++) neighboursFound[i] = 0;

	int p1, p2, p3;

	for(int t = 0; t<srf->NrOfTriangles*3; t+=3){
		p1 = srf->triangleSequence[t];
		p2 = srf->triangleSequence[t+1];
		p3 = srf->triangleSequence[t+2];

		//By adding the points in a certain order we keep the
		//winding of the triangles
		doubleNeighboursList[p1][neighboursFound[p1]] = p2;
		doubleNeighboursList[p1][neighboursFound[p1]+1] = p3;
		neighboursFound[p1] +=2;

		doubleNeighboursList[p2][neighboursFound[p2]] = p3;
		doubleNeighboursList[p2][neighboursFound[p2]+1] = p1;
		neighboursFound[p2] +=2;

		doubleNeighboursList[p3][neighboursFound[p3]] = p1;
		doubleNeighboursList[p3][neighboursFound[p3]+1] = p2;
		neighboursFound[p3] +=2;
	}

	return true;
}

bool eliminateNeighbours(srfStruct *srf, neighbours *neighb, int **doubleNeighboursList, int *nrNeighbours)
{
	int currentTarget, nn, lr;

	for(int i = 0; i<srf->NrOfVertices; i++){

		nn = nrNeighbours[i];
		neighb[i].nrOfNeighbours = nn;//nrNeighbours[i];
		int *neighbourLst = new int [nn];//[nrNeighbours[i]];
		neighbourLst[0] = doubleNeighboursList[i][0];
		neighbourLst[1] = doubleNeighboursList[i][1];

		//Added neighbours are set to zero!
		doubleNeighboursList[i][0] = 0;
		doubleNeighboursList[i][1] = 0;

		currentTarget = neighbourLst[1];

		for(int n = 0; n<(nn-2); n++) {

			// find current target in neighbour list
			for(int p = 2; p<nn*2; p++){
				if (doubleNeighboursList[i][p] == currentTarget){
					//p belongs to pair, find out if neighbour on left or right side is needed:
					lr = ((p % 2) > 0 ? (p-1) : (p+1));
					neighbourLst[2+n] = doubleNeighboursList[i][lr];
					currentTarget = neighbourLst[2+n];
					doubleNeighboursList[i][p] = 0;
					doubleNeighboursList[i][lr] = 0;
					break;
				}
			}
		}

		// vullen
		neighb[i].neighbourLst = neighbourLst;
	}

	return true;
}

int *convertLabelTableToMeshColors(gifti_image *gii, srfStruct *srf) { //giiLabelTable *giicolors, srfStruct *srf) {


	giiLabelTable *giicolors = &gii->labeltable;	
	bool success = true;
	int *meshcolors = new int[srf->NrOfVertices];
	for(int i = 0; i<srf->NrOfVertices; i++) meshcolors[i] = 0; 

	// 1. Check of nr of colors is equal
	if (giicolors == NULL) {

		qxLogText("No predefined colors present (NULL pointer). Colors will be default BrainVoyager colors (blue).");
		success = false;
		
	} else if (giicolors->length == 0) {
	
		qxLogText("No predefined colors present (length = 0). Colors will be default BrainVoyager colors (blue).");
		success = false;
	}

	// 2. Multiply RGBA floats to ints (0-255)
	if (success) {
		
		int rgbval;
		std::map<int, int> indexColorMapping;
		
		qxLogText("Converting GIFTI colors to BrainVoyager colors...");
		for (int i = 0; i<giicolors->length; i++){

			int red			= (giicolors->rgba[i*4]*255);
			int green		= (giicolors->rgba[i*4+1]*255);
			int blue		= (giicolors->rgba[i*4+2]*255);
			float alpha		= giicolors->rgba[i*4+3];
			int startval	= 1056964608;
			if (alpha == 0.0f) {
				rgbval = startval;
			} else {
				rgbval = startval + red * pow(16.0f,4) + green * pow(16.0f,2) + blue;
			}
			indexColorMapping[giicolors->key[i]] = rgbval;
		}
		
		int *vertexcolorblock = NULL; 		// if colors inside surface file, get DataArray with intent NIFTI_INTENT_LABEL
		int nrdataarray;
		
		for (int j=0; j<gii->numDA; j++) { 
			if (gii->darray[j]->intent == NIFTI_INTENT_LABEL || gii->darray[j]->intent == NIFTI_INTENT_NONE) {
				vertexcolorblock = (int*)gii->darray[j]->data; 
				nrdataarray = j;
				break;
			}
		} 
		
		qxLogText("Assigning converted colors to mesh...");
		// each mesh index refers to 1..n mesh color indices
		if (vertexcolorblock != NULL && (gii->darray[nrdataarray]->dims[0] <= srf->NrOfVertices)){
			for (int i = 0; i<gii->darray[nrdataarray]->dims[0]; i++){

				meshcolors[i] = indexColorMapping[vertexcolorblock[i]];
			}
		}
	}
	
	// 3. Return BV-colors
	return meshcolors;
} 

bool convertMeshColorsToLabelTable(srfStruct *srf) {
	
	
	// create gifti_image
	int dims[6] = {srf->NrOfVertices, 1, 1, 1, 1, 1};
	gifti_image *giftiimg = gifti_create_image(1, NIFTI_INTENT_LABEL, NIFTI_TYPE_UINT32, 6, dims, 0);
	giftiimg->numDA = 1;
	giftiimg->compressed = 0;
	// create pointers, define length in if-block
	float *rgba;
	int *key;
	char **label;
	
/*	typedef struct {
		int      length;      // length of each array, if allocated            
		int    * key;         // changed from index                 7 Mar 2010 
		char  ** label;
		float  * rgba;        // (optional) RGBA tuples (4*length, in [0,1.0]) : grootte 2 of grootte aantal kleuren
	} giiLabelTable;
*/
	
	int color, startval	= 1056964608;
	float red, green, blue, alpha;
	
	// check which possibility 
	if (srf->MeshColor[0] < startval) {
		
		giftiimg->labeltable.length = 2;
		key = new int [2];
		label = new char*[2];
		for(int i = 0; i<2; i++) label[i] = new char[256];// in docu: lengte is 256
		
		rgba = new float[2*4];
		
		//vullen
		key[0] = 0;
		key[1] = 1;
		sprintf(label[0], "Convex Curvature");
		sprintf(label[1], "Concave Curvature");
		rgba[0] = srf->ConvCurvRed;
		rgba[1] = srf->ConvCurvGreen;
		rgba[2] = srf->ConvCurvBlue;
		rgba[3] = srf->ConvCurvAlpha;
		rgba[4] = srf->ConcCurvRed;
		rgba[5] = srf->ConcCurvGreen;
		rgba[6] = srf->ConcCurvBlue;
		rgba[7] = srf->ConcCurvAlpha;
		
	//	An index value of 0 references the convex curvature color (stored after the normal vectors), 
	//	a value of 1 references the concave curvature color (stored after the convec curvature color).
		
	} else {
		
		// invoke functie die aantal kleuren in vertex colors array vindt.
		int length_colorlist;
		int *colorlist = colorcounter(srf->MeshColor, srf->NrOfVertices, &length_colorlist);
		
		// convert RGBA colors in huge number to 0.0-1.0.
		giftiimg->labeltable.length = length_colorlist;
		rgba = new float[length_colorlist*4];
		key = new int [length_colorlist];
		label = new char*[length_colorlist];
		for(int i = 0; i<length_colorlist; i++) label[i] = new char[256];// in docu: lengte is 256
		
		for (int i = 0; i<length_colorlist; i++){
			color = colorlist[i];
			color -= startval;
			red = color/pow(16.0f,4);
			green = (color-red*pow(16.0f,4))/pow(16.0f,2);
			blue = color-red*pow(16.0f,4)-green*pow(16.0f,2);
			red /= 255;
			green /= 255;
			blue /= 255;
			key[i] = color;
			sprintf(label[i], "Unknown");
			rgba[i*4] = red;
			rgba[1+i*4] = green;
			rgba[2+i*4] = blue;
			rgba[3+i*4] = 1.0f; //alpha value
		}
	}
	giftiimg->labeltable.rgba = rgba;
	giftiimg->labeltable.key = key;
	giftiimg->labeltable.label = label;

	// fill DataArray with colors.
	//int *vertexcolorblock = (int*)giftiimg->darray[0]->data;
	int *vertexcolorblock = new int[srf->NrOfVertices];
	
	// put colors in labeltable
	for (int i = 0; i<srf->NrOfVertices; i++){
		vertexcolorblock[i] = srf->MeshColor[i];
	}
	
	giftiimg->darray[0]->data = vertexcolorblock;
		
		
	// write to disk
	char *logtxt = new char[256];
	char *giimgname = exchangeExtension(srf->origName, ".label.gii");
	int success =  gifti_write_image(giftiimg, giimgname, 1);
	if (success == 0) {
		sprintf(logtxt, "Exported Color Label Table to GIfTI format, name:\n %s", giimgname);
		qxLogText(logtxt);
	} else {
		qxLogText("Export Color Label Table to GIfTI format did not succeed.");
	}
	
	return success;
	
}

// function that finds number of colors present in meshcolors array
int *colorcounter(int *meshcolors, int length_meshcolors, int *length_colorlist) {
	
		std::set<int> colorcontainer;
		for (int i=0; i<length_meshcolors; i++) {
			int currentcolor = meshcolors[i];
			if (colorcontainer.find(currentcolor) != colorcontainer.end()) {
				colorcontainer.insert(meshcolors[i]);
			}
		}
	
		int *colorlist = new int [colorcontainer.size()];
		*length_colorlist = colorcontainer.size();
	
		// copy set to array colorlist
		int i = 0;
		std::set<int>::iterator it;
		for (it=colorcontainer.begin(); it!=colorcontainer.end(); it++) {
			colorlist[i] = *it;
			i++;
		}
		 
	return colorlist;
}


/* Please note: 
 1. At the moment, this function only takes positive CC values.
 2. We assume that lowest lag is angle of 0 radians.
*/
std::complex<float> *crossCorrelationToComplex(smpStruct *smp) {

	// 1. splits in voor komma en achter komma (phase and magnitude)
	std::complex<float> *zdata = new std::complex<float>[smp->nrOfVertices];
	
	float anglePerLag = 2*PI/smp->srfStatMaps[0].nrOfLags;
	int minLag = smp->srfStatMaps[0].minLag;
	
	for (int i=0; i<smp->nrOfVertices; i++){

		// calculate phase (in radians) from lag
//		int lag		= (int)smp->srfStatMaps[0].mapData[i];
		int lag		= (int)(smp->srfStatMaps[0].mapData[i]/1000);
		
		float phase = anglePerLag*(lag - minLag); // phase
		float CC	= (smp->srfStatMaps[0].mapData[i]  - 1000*lag);// non-integer part of float data point.	
		// polar to rectangular (Cartesian)
		float x		= CC*cos(phase);
		float y		= CC*sin(phase);
		zdata[i]	= std::complex<float>(x, y);
		
		
	/*	char *log = new char[256]; 
		if (i<200) {
			sprintf(log, "Lag: %i, phase: %f, CC: %f, x: %f, y: %f, real(z): %f, imag(z): %f",
					lag, phase, CC, x, y, real(zdata[i]), imag(zdata[i]));
			qxLogText(log);		
		}*/
		
	
	}
		// the converse: rectangular to polar
	//	im = imag( F(9) );
	//	re = real( F(9) );
	//	phase_rad = atan2( im, re );		

/*Cross-correlation maps. Cross-correlation tests involve the computation of many correlations of a 
single time series with shifted versions of a reference function. 
At each shift or lag, a correlation value is computed. Lag values are allowed to have only positive values. 
In cross-correlation maps, each data point (4 byte float) saves two values simultaneously, the lag value 
which yielded the highest correlation and the (positive) correlation value obtained at this lag value. 
The saved value is defined as follows:
saved_val = lag_of_max_corr + (1.0 - max_corr) { if max_corr > 0.0
saved_val = -lag_of_max_corr + (1.0 + max_corr) { if max_corr < 0.0
		saved_val = 0.0 { otherwise
			Note that in cross-correlation maps, both max_corr and lag_of_max_corr values must be positive. 
			In the context of GLM maps (which internally use cross-correlation maps for "2 set color coding"), 
			negative values for max_corr are, however, allowed. To retrieve the lag and correlation value back 
 from the compound value, use code similar to this:
				max_corr = saved_val - (int)saved_val;
			lag_of_max_corr = (int)saved_val;
 */
	
	return zdata;
}

